@@ -25,7 +25,7 @@
 #include "geom.h"
 #include "stacking.h"
 #include "window.h"
-#include "render/color.h"
+#include "obrender/color.h"
 
 #include <glib.h>
 #include <X11/Xlib.h>
@@ -327,8 +327,6 @@ typedef void (*ObClientCallback)(ObClient *client, gpointer data);
 void client_add_destroy_notify(ObClientCallback func, gpointer data);
 void client_remove_destroy_notify(ObClientCallback func);
 
-/*! Manages all existing windows */
-void client_manage_all(void);
 /*! Manages a given window
   @param prompt This specifies an ObPrompt which is being managed.  It is
                 possible to manage Openbox-owned windows through this.
@@ -492,6 +490,11 @@ void client_find_resize_directional(ObClient *self, ObDirection side,
 */
 void client_fullscreen(ObClient *self, gboolean fs);
 
+/*! Determine if the window, using the given client-area, would be considered
+  as an "oldschool fullscreen" window, that is, if it is filling a whole
+  monitor. */
+gboolean client_is_oldfullscreen(const ObClient const *self, const Rect *area);
+
 /*! Iconifies or uniconifies the client window
   @param iconic true if the window should be iconified; false if it should be
                 restored.
@@ -654,6 +657,10 @@ RrImage* client_icon(ObClient *self);
   transient for */
 gboolean client_has_parent(ObClient *self);
 
+/*! Return TRUE if the client has some transient children, and FALSE otherwise.
+*/
+gboolean client_has_children(ObClient *self);
+
 /*! Searches a client's immediate parents for a focused window. The function
   does not check for the passed client, only for *ONE LEVEL* of its parents.
   If no focused parent is found, NULL is returned.
@@ -738,6 +745,11 @@ ObClient* client_under_pointer(void);
 
 gboolean client_has_group_siblings(ObClient *self);
 
+/*! Returns TRUE if the client has a transient child, a parent, or a
+  group member.  Returns FALSE otherwise.
+*/
+gboolean client_has_relative(ObClient *self);
+
 /*! Returns TRUE if the client is running on the same machine as Openbox */
 gboolean client_on_localhost(ObClient *self);
 
