@@ -27,8 +27,8 @@
 #include "frame.h"
 #include "moveresize.h"
 #include "event.h"
-#include "prop.h"
 #include "gettext.h"
+#include "obt/prop.h"
 
 #include <glib.h>
 
@@ -151,7 +151,7 @@ static void client_menu_execute(ObMenuEntry *e, ObMenuFrame *f,
 
         screen_pointer_pos(&x, &y);
         moveresize_start(c, x, y, 0,
-                         prop_atoms.net_wm_moveresize_move_keyboard);
+                         OBT_PROP_ATOM(NET_WM_MOVERESIZE_MOVE_KEYBOARD));
         break;
     case CLIENT_RESIZE:
         /* this needs to grab the keyboard so hide the menu */
@@ -160,7 +160,7 @@ static void client_menu_execute(ObMenuEntry *e, ObMenuFrame *f,
 
         screen_pointer_pos(&x, &y);
         moveresize_start(c, x, y, 0,
-                         prop_atoms.net_wm_moveresize_size_keyboard);
+                         OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_KEYBOARD));
         break;
     case CLIENT_CLOSE:
         client_close(c);
@@ -407,7 +407,9 @@ void client_menu_startup(void)
     e->data.normal.mask = ob_rr_theme->max_mask;
     set_icon_color(e);
 
-    menu_add_normal(menu, CLIENT_SHADE, _("_Roll up/down"), NULL, TRUE);
+    e = menu_add_normal(menu, CLIENT_SHADE, _("_Roll up/down"), NULL, TRUE);
+    e->data.normal.mask = ob_rr_theme->shade_mask;
+    set_icon_color(e);
 
     menu_add_normal(menu, CLIENT_DECORATE, _("Un/_Decorate"), NULL, TRUE);
 
