@@ -21,11 +21,13 @@
 
 #include <glib.h>
 
-void ob_debug_show_output(gboolean enable);
+void ob_debug_startup(void);
+void ob_debug_shutdown(void);
 
 void ob_debug(const gchar *a, ...);
 
 typedef enum {
+    OB_DEBUG_NORMAL,
     OB_DEBUG_FOCUS,
     OB_DEBUG_APP_BUGS,
     OB_DEBUG_SM,
