@@ -19,13 +19,12 @@
 
 #include "debug.h"
 #include "dock.h"
-#include "mainloop.h"
 #include "screen.h"
-#include "prop.h"
 #include "config.h"
 #include "grab.h"
 #include "openbox.h"
-#include "render/theme.h"
+#include "obrender/theme.h"
+#include "obt/prop.h"
 
 #define DOCK_EVENT_MASK (ButtonPressMask | ButtonReleaseMask | \
                          EnterWindowMask | LeaveWindowMask)
@@ -34,6 +33,8 @@
                               ButtonMotionMask)
 
 static ObDock *dock;
+static guint show_timeout_id;
+static guint hide_timeout_id;
 
 StrutPartial dock_strut;
 
@@ -51,6 +52,9 @@ static void dock_app_grab_button(ObDockApp *app, gboolean grab)
     }
 }
 
+static guint window_hash(Window *w) { return *w; }
+static gboolean window_comp(Window *w1, Window *w2) { return *w1 == *w2; }
+
 void dock_startup(gboolean reconfig)
 {
     XSetWindowAttributes attrib;
@@ -58,12 +62,12 @@ void dock_startup(gboolean reconfig)
     if (reconfig) {
         GList *it;
 
-        XSetWindowBorder(ob_display, dock->frame,
+        XSetWindowBorder(obt_display, dock->frame,
                          RrColorPixel(ob_rr_theme->osd_border_color));
-        XSetWindowBorderWidth(ob_display, dock->frame, ob_rr_theme->obwidth);
+        XSetWindowBorderWidth(obt_display, dock->frame, ob_rr_theme->obwidth);
 
         RrAppearanceFree(dock->a_frame);
-        dock->a_frame = RrAppearanceCopy(ob_rr_theme->osd_hilite_bg);
+        dock->a_frame = RrAppearanceCopy(ob_rr_theme->osd_bg);
 
         stacking_add(DOCK_AS_WINDOW(dock));
 
@@ -78,31 +82,34 @@ void dock_startup(gboolean reconfig)
     STRUT_PARTIAL_SET(dock_strut, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0);
 
-    dock = g_new0(ObDock, 1);
-    dock->obwin.type = Window_Dock;
+    dock = g_slice_new0(ObDock);
+    dock->obwin.type = OB_WINDOW_CLASS_DOCK;
 
     dock->hidden = TRUE;
 
+    dock->dock_map = g_hash_table_new((GHashFunc)window_hash,
+                                      (GEqualFunc)window_comp);
+
     attrib.event_mask = DOCK_EVENT_MASK;
     attrib.override_redirect = True;
     attrib.do_not_propagate_mask = DOCK_NOPROPAGATEMASK;
-    dock->frame = XCreateWindow(ob_display, RootWindow(ob_display, ob_screen),
+    dock->frame = XCreateWindow(obt_display, obt_root(ob_screen),
                                 0, 0, 1, 1, 0,
                                 RrDepth(ob_rr_inst), InputOutput,
                                 RrVisual(ob_rr_inst),
                                 CWOverrideRedirect | CWEventMask |
                                 CWDontPropagate,
                                 &attrib);
-    dock->a_frame = RrAppearanceCopy(ob_rr_theme->osd_hilite_bg);
-    XSetWindowBorder(ob_display, dock->frame,
+    dock->a_frame = RrAppearanceCopy(ob_rr_theme->osd_bg);
+    XSetWindowBorder(obt_display, dock->frame,
                      RrColorPixel(ob_rr_theme->osd_border_color));
-    XSetWindowBorderWidth(ob_display, dock->frame, ob_rr_theme->obwidth);
+    XSetWindowBorderWidth(obt_display, dock->frame, ob_rr_theme->obwidth);
 
     /* Setting the window type so xcompmgr can tell what it is */
-    PROP_SET32(dock->frame, net_wm_window_type, atom,
-               prop_atoms.net_wm_window_type_dock);
+    OBT_PROP_SET32(dock->frame, NET_WM_WINDOW_TYPE, ATOM,
+                   OBT_PROP_ATOM(NET_WM_WINDOW_TYPE_DOCK));
 
-    g_hash_table_insert(window_map, &dock->frame, dock);
+    window_add(&dock->frame, DOCK_AS_WINDOW(dock));
     stacking_add(DOCK_AS_WINDOW(dock));
 }
 
@@ -118,25 +125,27 @@ void dock_shutdown(gboolean reconfig)
         return;
     }
 
-    XDestroyWindow(ob_display, dock->frame);
+    g_hash_table_destroy(dock->dock_map);
+
+    XDestroyWindow(obt_display, dock->frame);
     RrAppearanceFree(dock->a_frame);
-    g_hash_table_remove(window_map, &dock->frame);
+    window_remove(dock->frame);
     stacking_remove(dock);
+    g_slice_free(ObDock, dock);
+    dock = NULL;
 }
 
-void dock_add(Window win, XWMHints *wmhints)
+void dock_manage(Window icon_win, Window name_win)
 {
     ObDockApp *app;
     XWindowAttributes attrib;
     gchar **data;
 
-    app = g_new0(ObDockApp, 1);
-    app->obwin.type = Window_DockApp;
-    app->win = win;
-    app->icon_win = (wmhints->flags & IconWindowHint) ?
-        wmhints->icon_window : win;
+    app = g_slice_new0(ObDockApp);
+    app->name_win = name_win;
+    app->icon_win = icon_win;
 
-    if (PROP_GETSS(app->win, wm_class, locale, &data)) {
+    if (OBT_PROP_GETSS_TYPE(app->name_win, WM_CLASS, STRING_NO_CC, &data)) {
         if (data[0]) {
             app->name = g_strdup(data[0]);
             if (data[1])
@@ -148,7 +157,7 @@ void dock_add(Window win, XWMHints *wmhints)
     if (app->name == NULL) app->name = g_strdup("");
     if (app->class == NULL) app->class = g_strdup("");
 
-    if (XGetWindowAttributes(ob_display, app->icon_win, &attrib)) {
+    if (XGetWindowAttributes(obt_display, app->icon_win, &attrib)) {
         app->w = attrib.width;
         app->h = attrib.height;
     } else {
@@ -156,9 +165,10 @@ void dock_add(Window win, XWMHints *wmhints)
     }
 
     dock->dock_apps = g_list_append(dock->dock_apps, app);
+    g_hash_table_insert(dock->dock_map, &app->icon_win, app);
     dock_configure();
 
-    XReparentWindow(ob_display, app->icon_win, dock->frame, app->x, app->y);
+    XReparentWindow(obt_display, app->icon_win, dock->frame, app->x, app->y);
     /*
       This is the same case as in frame.c for client windows. When Openbox is
       starting, the window is already mapped so we see unmap events occur for
@@ -168,55 +178,57 @@ void dock_add(Window win, XWMHints *wmhints)
     */
     if (ob_state() == OB_STATE_STARTING)
         app->ignore_unmaps += 2;
+    XChangeSaveSet(obt_display, app->icon_win, SetModeInsert);
+    XMapWindow(obt_display, app->icon_win);
 
-    if (app->win != app->icon_win) {
-        /* have to map it so that it can be re-managed on a restart */
-        XMoveWindow(ob_display, app->win, -1000, -1000);
-        XMapWindow(ob_display, app->win);
+    if (app->name_win != app->icon_win) {
+        XReparentWindow(obt_display, app->name_win, dock->frame, -1000, -1000);
+        XChangeSaveSet(obt_display, app->name_win, SetModeInsert);
+        XMapWindow(obt_display, app->name_win);
     }
-    XMapWindow(ob_display, app->icon_win);
-    XSync(ob_display, False);
 
-    /* specify that if we exit, the window should not be destroyed and should
-       be reparented back to root automatically */
-    XChangeSaveSet(ob_display, app->icon_win, SetModeInsert);
-    XSelectInput(ob_display, app->icon_win, DOCKAPP_EVENT_MASK);
+    XSync(obt_display, False);
+
+    XSelectInput(obt_display, app->icon_win, DOCKAPP_EVENT_MASK);
 
     dock_app_grab_button(app, TRUE);
 
-    g_hash_table_insert(window_map, &app->icon_win, app);
+    ob_debug("Managed Dock App: 0x%lx 0x%lx (%s)",
+             app->icon_win, app->name_win, app->class);
 
-    ob_debug("Managed Dock App: 0x%lx (%s)\n", app->icon_win, app->class);
+    grab_server(FALSE);
 }
 
-void dock_remove_all(void)
+void dock_unmanage_all(void)
 {
     while (dock->dock_apps)
-        dock_remove(dock->dock_apps->data, TRUE);
+        dock_unmanage(dock->dock_apps->data, TRUE);
 }
 
-void dock_remove(ObDockApp *app, gboolean reparent)
+void dock_unmanage(ObDockApp *app, gboolean reparent)
 {
     dock_app_grab_button(app, FALSE);
-    XSelectInput(ob_display, app->icon_win, NoEventMask);
+    XSelectInput(obt_display, app->icon_win, NoEventMask);
     /* remove the window from our save set */
-    XChangeSaveSet(ob_display, app->icon_win, SetModeDelete);
-    XSync(ob_display, False);
+    XChangeSaveSet(obt_display, app->icon_win, SetModeDelete);
+    XSync(obt_display, False);
 
-    g_hash_table_remove(window_map, &app->icon_win);
-
-    if (reparent)
-        XReparentWindow(ob_display, app->icon_win,
-                        RootWindow(ob_display, ob_screen), app->x, app->y);
+    if (reparent) {
+        XReparentWindow(obt_display, app->icon_win, obt_root(ob_screen), 0, 0);
+        if (app->name_win != app->icon_win)
+            XReparentWindow(obt_display, app->name_win,
+                            obt_root(ob_screen), 0, 0);
+    }
 
     dock->dock_apps = g_list_remove(dock->dock_apps, app);
+    g_hash_table_remove(dock->dock_map, &app->icon_win);
     dock_configure();
 
-    ob_debug("Unmanaged Dock App: 0x%lx (%s)\n", app->icon_win, app->class);
+    ob_debug("Unmanaged Dock App: 0x%lx (%s)", app->icon_win, app->class);
 
     g_free(app->name);
     g_free(app->class);
-    g_free(app);
+    g_slice_free(ObDockApp, app);
 }
 
 void dock_configure(void)
@@ -226,7 +238,7 @@ void dock_configure(void)
     gint gravity;
     gint l, r, t, b;
     gint strw, strh;
-    Rect *a;
+    const Rect *a;
     gint hidesize;
 
     RrMargins(dock->a_frame, &l, &t, &r, &b);
@@ -273,7 +285,7 @@ void dock_configure(void)
             break;
         }
 
-        XMoveWindow(ob_display, app->icon_win, app->x, app->y);
+        XMoveWindow(obt_display, app->icon_win, app->x, app->y);
     }
 
     /* used for calculating offsets */
@@ -523,14 +535,14 @@ void dock_configure(void)
         g_assert(dock->area.width > 0);
         g_assert(dock->area.height > 0);
 
-        XMoveResizeWindow(ob_display, dock->frame, dock->area.x, dock->area.y,
+        XMoveResizeWindow(obt_display, dock->frame, dock->area.x, dock->area.y,
                           dock->area.width, dock->area.height);
 
         RrPaint(dock->a_frame, dock->frame, dock->area.width,
                 dock->area.height);
-        XMapWindow(ob_display, dock->frame);
+        XMapWindow(obt_display, dock->frame);
     } else
-        XUnmapWindow(ob_display, dock->frame);
+        XUnmapWindow(obt_display, dock->frame);
 
     /* but they are useful outside of this function! but don't add it if the
        dock is actually not visible */
@@ -539,9 +551,9 @@ void dock_configure(void)
         dock->area.height += ob_rr_theme->obwidth * 2;
     }
 
+    /* screen_resize() depends on this function to call screen_update_areas(),
+       so if this changes, also update screen_resize(). */
     screen_update_areas();
-
-    g_free(a);
 }
 
 void dock_app_configure(ObDockApp *app, gint w, gint h)
@@ -620,15 +632,19 @@ static gboolean hide_timeout(gpointer data)
     dock->hidden = TRUE;
     dock_configure();
 
+    hide_timeout_id = 0;
+
     return FALSE; /* don't repeat */
 }
 
 static gboolean show_timeout(gpointer data)
 {
-    /* hide */
+    /* show */
     dock->hidden = FALSE;
     dock_configure();
 
+    show_timeout_id = 0;
+
     return FALSE; /* don't repeat */
 }
 
@@ -636,19 +652,21 @@ void dock_hide(gboolean hide)
 {
     if (!hide) {
         if (dock->hidden && config_dock_hide) {
-            ob_main_loop_timeout_add(ob_main_loop,
-                                     config_dock_show_delay * 1000,
-                                     show_timeout, NULL, g_direct_equal, NULL);
-        } else if (!dock->hidden && config_dock_hide) {
-            ob_main_loop_timeout_remove(ob_main_loop, hide_timeout);
+            show_timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT,
+                                                 config_dock_show_delay,
+                                                 show_timeout, NULL, NULL);
+        } else if (!dock->hidden && config_dock_hide && hide_timeout_id) {
+            if (hide_timeout_id) g_source_remove(hide_timeout_id);
+            hide_timeout_id = 0;
         }
     } else {
         if (!dock->hidden && config_dock_hide) {
-            ob_main_loop_timeout_add(ob_main_loop,
-                                     config_dock_hide_delay * 1000,
-                                     hide_timeout, NULL, g_direct_equal, NULL);
-        } else if (dock->hidden && config_dock_hide) {
-            ob_main_loop_timeout_remove(ob_main_loop, show_timeout);
+            hide_timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT,
+                                                 config_dock_show_delay,
+                                                 hide_timeout, NULL, NULL);
+        } else if (dock->hidden && config_dock_hide && show_timeout_id) {
+            if (show_timeout_id) g_source_remove(show_timeout_id);
+            show_timeout_id = 0;
         }
     }
 }
@@ -658,3 +676,18 @@ void dock_get_area(Rect *a)
     RECT_SET(*a, dock->area.x, dock->area.y,
              dock->area.width, dock->area.height);
 }
+
+void dock_raise_dock(void)
+{
+    stacking_raise(DOCK_AS_WINDOW(dock));
+}
+
+void dock_lower_dock(void)
+{
+    stacking_lower(DOCK_AS_WINDOW(dock));
+}
+
+ObDockApp* dock_find_dockapp(Window xwin)
+{
+    return g_hash_table_lookup(dock->dock_map, &xwin);
+}
