@@ -20,7 +20,6 @@
 #include "debug.h"
 #include "menu.h"
 #include "openbox.h"
-#include "mainloop.h"
 #include "stacking.h"
 #include "grab.h"
 #include "client.h"
@@ -35,7 +34,8 @@
 #include "client_list_menu.h"
 #include "client_list_combined_menu.h"
 #include "gettext.h"
-#include "parser/parse.h"
+#include "obt/xml.h"
+#include "obt/paths.h"
 
 typedef struct _ObMenuParseState ObMenuParseState;
 
@@ -46,26 +46,21 @@ struct _ObMenuParseState
 };
 
 static GHashTable *menu_hash = NULL;
-static ObParseInst *menu_parse_inst;
+static ObtXmlInst *menu_parse_inst;
 static ObMenuParseState menu_parse_state;
 static gboolean menu_can_hide = FALSE;
+static guint menu_timeout_id = 0;
 
 static void menu_destroy_hash_value(ObMenu *self);
-static void parse_menu_item(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
-                            gpointer data);
-static void parse_menu_separator(ObParseInst *i,
-                                 xmlDocPtr doc, xmlNodePtr node,
-                                 gpointer data);
-static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
-                       gpointer data);
+static void parse_menu_item(xmlNodePtr node, gpointer data);
+static void parse_menu_separator(xmlNodePtr node, gpointer data);
+static void parse_menu(xmlNodePtr node, gpointer data);
 static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
                                gchar **strippedlabel, guint *position,
                                gboolean *always_show);
 
 void menu_startup(gboolean reconfig)
 {
-    xmlDocPtr doc;
-    xmlNodePtr node;
     gboolean loaded = FALSE;
     GSList *it;
 
@@ -76,29 +71,37 @@ void menu_startup(gboolean reconfig)
     client_list_combined_menu_startup(reconfig);
     client_menu_startup();
 
-    menu_parse_inst = parse_startup();
+    menu_parse_inst = obt_xml_instance_new();
 
     menu_parse_state.parent = NULL;
     menu_parse_state.pipe_creator = NULL;
-    parse_register(menu_parse_inst, "menu", parse_menu, &menu_parse_state);
-    parse_register(menu_parse_inst, "item", parse_menu_item,
-                   &menu_parse_state);
-    parse_register(menu_parse_inst, "separator",
-                   parse_menu_separator, &menu_parse_state);
+    obt_xml_register(menu_parse_inst, "menu", parse_menu, &menu_parse_state);
+    obt_xml_register(menu_parse_inst, "item", parse_menu_item,
+                     &menu_parse_state);
+    obt_xml_register(menu_parse_inst, "separator",
+                       parse_menu_separator, &menu_parse_state);
 
     for (it = config_menu_files; it; it = g_slist_next(it)) {
-        if (parse_load_menu(it->data, &doc, &node)) {
+        if (obt_xml_load_config_file(menu_parse_inst,
+                                     "openbox",
+                                     it->data,
+                                     "openbox_menu"))
+        {
             loaded = TRUE;
-            parse_tree(menu_parse_inst, doc, node->children);
-            xmlFreeDoc(doc);
+            obt_xml_tree_from_root(menu_parse_inst);
+            obt_xml_close(menu_parse_inst);
         } else
             g_message(_("Unable to find a valid menu file \"%s\""),
                       (const gchar*)it->data);
     }
     if (!loaded) {
-        if (parse_load_menu("menu.xml", &doc, &node)) {
-            parse_tree(menu_parse_inst, doc, node->children);
-            xmlFreeDoc(doc);
+        if (obt_xml_load_config_file(menu_parse_inst,
+                                     "openbox",
+                                     "menu.xml",
+                                     "openbox_menu"))
+        {
+            obt_xml_tree_from_root(menu_parse_inst);
+            obt_xml_close(menu_parse_inst);
         } else
             g_message(_("Unable to find a valid menu file \"%s\""),
                       "menu.xml");
@@ -109,7 +112,7 @@ void menu_startup(gboolean reconfig)
 
 void menu_shutdown(gboolean reconfig)
 {
-    parse_shutdown(menu_parse_inst);
+    obt_xml_instance_unref(menu_parse_inst);
     menu_parse_inst = NULL;
 
     client_list_menu_shutdown(reconfig);
@@ -143,8 +146,6 @@ void menu_clear_pipe_caches(void)
 
 void menu_pipe_execute(ObMenu *self)
 {
-    xmlDocPtr doc;
-    xmlNodePtr node;
     gchar *output;
     GError *err = NULL;
 
@@ -160,13 +161,13 @@ void menu_pipe_execute(ObMenu *self)
         return;
     }
 
-    if (parse_load_mem(output, strlen(output),
-                       "openbox_pipe_menu", &doc, &node))
+    if (obt_xml_load_mem(menu_parse_inst, output, strlen(output),
+                         "openbox_pipe_menu"))
     {
         menu_parse_state.pipe_creator = self;
         menu_parse_state.parent = self;
-        parse_tree(menu_parse_inst, doc, node->children);
-        xmlFreeDoc(doc);
+        obt_xml_tree_from_root(menu_parse_inst);
+        obt_xml_close(menu_parse_inst);
     } else {
         g_message(_("Invalid output from pipe-menu \"%s\""), self->execute);
     }
@@ -266,38 +267,53 @@ static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
     return shortcut;
 }
 
-static void parse_menu_item(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
-                            gpointer data)
+static void parse_menu_item(xmlNodePtr node,  gpointer data)
 {
     ObMenuParseState *state = data;
     gchar *label;
+    gchar *icon;
+    ObMenuEntry *e;
 
     if (state->parent) {
-        if (parse_attr_string("label", node, &label)) {
+        /* Don't try to extract "icon" attribute if icons in user-defined
+           menus are not enabled. */
+
+        if (obt_xml_attr_string(node, "label", &label)) {
+            xmlNodePtr c;
             GSList *acts = NULL;
 
-            for (node = node->children; node; node = node->next)
-                if (!xmlStrcasecmp(node->name, (const xmlChar*) "action")) {
-                    ObActionsAct *a = actions_parse(i, doc, node);
-                    if (a)
-                        acts = g_slist_append(acts, a);
-                }
-            menu_add_normal(state->parent, -1, label, acts, TRUE);
+            c = obt_xml_find_node(node->children, "action");
+            while (c) {
+                ObActionsAct *action = actions_parse(c);
+                if (action)
+                    acts = g_slist_append(acts, action);
+                c = obt_xml_find_node(node->next, "action");
+            }
+            e = menu_add_normal(state->parent, -1, label, acts, TRUE);
+            
+            if (config_menu_show_icons &&
+                obt_xml_attr_string(node, "icon", &icon))
+            {
+                e->data.normal.icon = RrImageNewFromName(ob_rr_icons, icon);
+
+                if (e->data.normal.icon)
+                    e->data.normal.icon_alpha = 0xff;
+
+                g_free(icon);
+            }
             g_free(label);
         }
     }
 }
 
-static void parse_menu_separator(ObParseInst *i,
-                                 xmlDocPtr doc, xmlNodePtr node,
-                                 gpointer data)
+static void parse_menu_separator(xmlNodePtr node, gpointer data)
 {
     ObMenuParseState *state = data;
 
     if (state->parent) {
         gchar *label;
 
-        if (!parse_attr_string("label", node, &label))
+        if (!obt_xml_attr_string(node, "label", &label))
             label = NULL;
 
         menu_add_separator(state->parent, -1, label);
@@ -305,37 +321,50 @@ static void parse_menu_separator(ObParseInst *i,
     }
 }
 
-static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
-                       gpointer data)
+static void parse_menu(xmlNodePtr node, gpointer data)
 {
     ObMenuParseState *state = data;
     gchar *name = NULL, *title = NULL, *script = NULL;
     ObMenu *menu;
+    ObMenuEntry *e;
+    gchar *icon;
 
-    if (!parse_attr_string("id", node, &name))
+    if (!obt_xml_attr_string(node, "id", &name))
         goto parse_menu_fail;
 
     if (!g_hash_table_lookup(menu_hash, name)) {
-        if (!parse_attr_string("label", node, &title))
+        if (!obt_xml_attr_string(node, "label", &title))
             goto parse_menu_fail;
 
         if ((menu = menu_new(name, title, TRUE, NULL))) {
             menu->pipe_creator = state->pipe_creator;
-            if (parse_attr_string("execute", node, &script)) {
-                menu->execute = parse_expand_tilde(script);
+            if (obt_xml_attr_string(node, "execute", &script)) {
+                menu->execute = obt_paths_expand_tilde(script);
             } else {
                 ObMenu *old;
 
                 old = state->parent;
                 state->parent = menu;
-                parse_tree(i, doc, node->children);
+                obt_xml_tree(menu_parse_inst, node->children);
                 state->parent = old;
             }
         }
     }
 
-    if (state->parent)
-        menu_add_submenu(state->parent, -1, name);
+    if (state->parent) {
+        e = menu_add_submenu(state->parent, -1, name);
+
+        if (config_menu_show_icons &&
+            obt_xml_attr_string(node, "icon", &icon))
+        {
+            e->data.submenu.icon = RrImageNewFromName(ob_rr_icons, icon);
+
+            if (e->data.submenu.icon)
+                e->data.submenu.icon_alpha = 0xff;
+
+            g_free(icon);
+        }
+    }
 
 parse_menu_fail:
     g_free(name);
@@ -348,7 +377,7 @@ ObMenu* menu_new(const gchar *name, const gchar *title,
 {
     ObMenu *self;
 
-    self = g_new0(ObMenu, 1);
+    self = g_slice_new0(ObMenu);
     self->name = g_strdup(name);
     self->data = data;
 
@@ -367,7 +396,7 @@ ObMenu* menu_new(const gchar *name, const gchar *title,
 
        more_menu->more_menu will always be NULL, since there is only 1 for
        each menu. */
-    self->more_menu = g_new0(ObMenu, 1);
+    self->more_menu = g_slice_new0(ObMenu);
     self->more_menu->name = _("More...");
     self->more_menu->title = _("More...");
     self->more_menu->data = data;
@@ -397,9 +426,9 @@ static void menu_destroy_hash_value(ObMenu *self)
     g_free(self->name);
     g_free(self->title);
     g_free(self->execute);
-    g_free(self->more_menu);
+    g_slice_free(ObMenu, self->more_menu);
 
-    g_free(self);
+    g_slice_free(ObMenu, self);
 }
 
 void menu_free(ObMenu *menu)
@@ -411,6 +440,7 @@ void menu_free(ObMenu *menu)
 static gboolean menu_hide_delay_func(gpointer data)
 {
     menu_can_hide = TRUE;
+    menu_timeout_id = 0;
     return FALSE; /* no repeat */
 }
 
@@ -460,10 +490,11 @@ void menu_show(gchar *name, gint x, gint y, gboolean mouse, ObClient *client)
             menu_can_hide = TRUE;
         else {
             menu_can_hide = FALSE;
-            ob_main_loop_timeout_add(ob_main_loop,
-                                     config_menu_hide_delay * 1000,
-                                     menu_hide_delay_func,
-                                     NULL, g_direct_equal, NULL);
+            if (menu_timeout_id) g_source_remove(menu_timeout_id);
+            menu_timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT,
+                                                 config_menu_hide_delay,
+                                                 menu_hide_delay_func,
+                                                 NULL, NULL);
         }
     }
 }
@@ -479,7 +510,7 @@ static ObMenuEntry* menu_entry_new(ObMenu *menu, ObMenuEntryType type, gint id)
 
     g_assert(menu);
 
-    self = g_new0(ObMenuEntry, 1);
+    self = g_slice_new0(ObMenuEntry);
     self->ref = 1;
     self->type = type;
     self->menu = menu;
@@ -517,6 +548,7 @@ void menu_entry_unref(ObMenuEntry *self)
             }
             break;
         case OB_MENU_ENTRY_TYPE_SUBMENU:
+            RrImageUnref(self->data.submenu.icon);
             g_free(self->data.submenu.name);
             break;
         case OB_MENU_ENTRY_TYPE_SEPARATOR:
@@ -524,7 +556,7 @@ void menu_entry_unref(ObMenuEntry *self)
             break;
         }
 
-        g_free(self);
+        g_slice_free(ObMenuEntry, self);
     }
 }
 
@@ -628,6 +660,11 @@ void menu_set_execute_func(ObMenu *self, ObMenuExecuteFunc func)
     self->more_menu->execute_func = func; /* keep it in sync */
 }
 
+void menu_set_cleanup_func(ObMenu *self, ObMenuCleanupFunc func)
+{
+    self->cleanup_func = func;
+}
+
 void menu_set_destroy_func(ObMenu *self, ObMenuDestroyFunc func)
 {
     self->destroy_func = func;
