@@ -21,14 +21,14 @@
 #include "client.h"
 #include "menu.h"
 #include "screen.h"
-#include "prop.h"
 #include "actions.h"
 #include "event.h"
 #include "grab.h"
 #include "openbox.h"
-#include "mainloop.h"
 #include "config.h"
-#include "render/theme.h"
+#include "obt/prop.h"
+#include "obt/keyboard.h"
+#include "obrender/theme.h"
 
 #define PADDING 2
 #define MAX_MENU_WIDTH 400
@@ -44,6 +44,8 @@ GList *menu_frame_visible;
 GHashTable *menu_frame_map;
 
 static RrAppearance *a_sep;
+static guint submenu_show_timer = 0;
+static guint submenu_hide_timer = 0;
 
 static ObMenuEntryFrame* menu_entry_frame_new(ObMenuEntry *entry,
                                               ObMenuFrame *frame);
@@ -57,7 +59,7 @@ static gboolean submenu_hide_timeout(gpointer data);
 static Window createWindow(Window parent, gulong mask,
                            XSetWindowAttributes *attrib)
 {
-    return XCreateWindow(ob_display, parent, 0, 0, 1, 1, 0,
+    return XCreateWindow(obt_display, parent, 0, 0, 1, 1, 0,
                          RrDepth(ob_rr_inst), InputOutput,
                          RrVisual(ob_rr_inst), mask, attrib);
 }
@@ -108,8 +110,8 @@ ObMenuFrame* menu_frame_new(ObMenu *menu, guint show_from, ObClient *client)
     ObMenuFrame *self;
     XSetWindowAttributes attr;
 
-    self = g_new0(ObMenuFrame, 1);
-    self->type = Window_Menu;
+    self = g_slice_new0(ObMenuFrame);
+    self->obwin.type = OB_WINDOW_CLASS_MENUFRAME;
     self->menu = menu;
     self->selected = NULL;
     self->client = client;
@@ -117,20 +119,21 @@ ObMenuFrame* menu_frame_new(ObMenu *menu, guint show_from, ObClient *client)
     self->show_from = show_from;
 
     attr.event_mask = FRAME_EVENTMASK;
-    self->window = createWindow(RootWindow(ob_display, ob_screen),
+    self->window = createWindow(obt_root(ob_screen),
                                 CWEventMask, &attr);
 
     /* make it a popup menu type window */
-    PROP_SET32(self->window, net_wm_window_type, atom,
-               prop_atoms.net_wm_window_type_popup_menu);
+    OBT_PROP_SET32(self->window, NET_WM_WINDOW_TYPE, ATOM,
+                   OBT_PROP_ATOM(NET_WM_WINDOW_TYPE_POPUP_MENU));
 
-    XSetWindowBorderWidth(ob_display, self->window, ob_rr_theme->mbwidth);
-    XSetWindowBorder(ob_display, self->window,
+    XSetWindowBorderWidth(obt_display, self->window, ob_rr_theme->mbwidth);
+    XSetWindowBorder(obt_display, self->window,
                      RrColorPixel(ob_rr_theme->menu_border_color));
 
     self->a_items = RrAppearanceCopy(ob_rr_theme->a_menu);
 
-    stacking_add(MENU_AS_WINDOW(self));
+    window_add(&self->window, MENUFRAME_AS_WINDOW(self));
+    stacking_add(MENUFRAME_AS_WINDOW(self));
 
     return self;
 }
@@ -143,23 +146,31 @@ void menu_frame_free(ObMenuFrame *self)
             self->entries = g_list_delete_link(self->entries, self->entries);
         }
 
-        stacking_remove(MENU_AS_WINDOW(self));
+        stacking_remove(MENUFRAME_AS_WINDOW(self));
+        window_remove(self->window);
 
         RrAppearanceFree(self->a_items);
 
-        XDestroyWindow(ob_display, self->window);
+        XDestroyWindow(obt_display, self->window);
 
-        g_free(self);
+        g_slice_free(ObMenuFrame, self);
     }
 }
 
+ObtIC* menu_frame_ic(ObMenuFrame *self)
+{
+    /* menus are always used through a grab right now, so they can always use
+       the grab input context */
+    return grab_input_context();
+}
+
 static ObMenuEntryFrame* menu_entry_frame_new(ObMenuEntry *entry,
                                               ObMenuFrame *frame)
 {
     ObMenuEntryFrame *self;
     XSetWindowAttributes attr;
 
-    self = g_new0(ObMenuEntryFrame, 1);
+    self = g_slice_new0(ObMenuEntryFrame);
     self->entry = entry;
     self->frame = frame;
 
@@ -170,7 +181,8 @@ static ObMenuEntryFrame* menu_entry_frame_new(ObMenuEntry *entry,
     self->text = createWindow(self->window, 0, NULL);
     g_hash_table_insert(menu_frame_map, &self->window, self);
     g_hash_table_insert(menu_frame_map, &self->text, self);
-    if (entry->type == OB_MENU_ENTRY_TYPE_NORMAL) {
+    if ((entry->type == OB_MENU_ENTRY_TYPE_NORMAL) ||
+        (entry->type == OB_MENU_ENTRY_TYPE_SUBMENU)) {
         self->icon = createWindow(self->window, 0, NULL);
         g_hash_table_insert(menu_frame_map, &self->icon, self);
     }
@@ -179,8 +191,10 @@ static ObMenuEntryFrame* menu_entry_frame_new(ObMenuEntry *entry,
         g_hash_table_insert(menu_frame_map, &self->bullet, self);
     }
 
-    XMapWindow(ob_display, self->window);
-    XMapWindow(ob_display, self->text);
+    XMapWindow(obt_display, self->window);
+    XMapWindow(obt_display, self->text);
+
+    window_add(&self->window, MENUFRAME_AS_WINDOW(self->frame));
 
     return self;
 }
@@ -190,20 +204,23 @@ static void menu_entry_frame_free(ObMenuEntryFrame *self)
     if (self) {
         menu_entry_unref(self->entry);
 
-        XDestroyWindow(ob_display, self->text);
-        XDestroyWindow(ob_display, self->window);
+        window_remove(self->window);
+
+        XDestroyWindow(obt_display, self->text);
+        XDestroyWindow(obt_display, self->window);
         g_hash_table_remove(menu_frame_map, &self->text);
         g_hash_table_remove(menu_frame_map, &self->window);
-        if (self->entry->type == OB_MENU_ENTRY_TYPE_NORMAL) {
-            XDestroyWindow(ob_display, self->icon);
+        if ((self->entry->type == OB_MENU_ENTRY_TYPE_NORMAL) ||
+            (self->entry->type == OB_MENU_ENTRY_TYPE_SUBMENU)) {
+            XDestroyWindow(obt_display, self->icon);
             g_hash_table_remove(menu_frame_map, &self->icon);
         }
         if (self->entry->type == OB_MENU_ENTRY_TYPE_SUBMENU) {
-            XDestroyWindow(ob_display, self->bullet);
+            XDestroyWindow(obt_display, self->bullet);
             g_hash_table_remove(menu_frame_map, &self->bullet);
         }
 
-        g_free(self);
+        g_slice_free(ObMenuEntryFrame, self);
     }
 }
 
@@ -211,7 +228,7 @@ void menu_frame_move(ObMenuFrame *self, gint x, gint y)
 {
     RECT_SET_POINT(self->area, x, y);
     self->monitor = screen_find_monitor_point(x, y);
-    XMoveWindow(ob_display, self->window, self->area.x, self->area.y);
+    XMoveWindow(obt_display, self->window, self->area.x, self->area.y);
 }
 
 static void menu_frame_place_topmenu(ObMenuFrame *self, gint *x, gint *y)
@@ -308,12 +325,19 @@ static void menu_frame_place_submenu(ObMenuFrame *self, gint *x, gint *y)
 void menu_frame_move_on_screen(ObMenuFrame *self, gint x, gint y,
                                gint *dx, gint *dy)
 {
-    Rect *a = NULL;
-    gint pos, half;
+    const Rect *a = NULL;
+    Rect search = self->area;
+    gint pos, half, monitor;
 
     *dx = *dy = 0;
+    RECT_SET_POINT(search, x, y);
 
-    a = screen_physical_area_monitor(screen_find_monitor_point(x, y));
+    if (self->parent)
+        monitor = self->parent->monitor;
+    else
+        monitor = screen_find_monitor(&search);
+
+    a = screen_physical_area_monitor(monitor);
 
     half = g_list_length(self->entries) / 2;
     pos = g_list_index(self->entries, self->selected);
@@ -332,8 +356,6 @@ void menu_frame_move_on_screen(ObMenuFrame *self, gint x, gint y,
         *dx = MAX(*dx, a->x - x);
         *dy = MAX(*dy, a->y - y);
     }
-
-    g_free(a);
 }
 
 static void menu_entry_frame_render(ObMenuEntryFrame *self)
@@ -373,7 +395,7 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
     }
 
     RECT_SET_SIZE(self->area, self->frame->inner_w, th);
-    XResizeWindow(ob_display, self->window,
+    XResizeWindow(obt_display, self->window,
                   self->area.width, self->area.height);
     item_a->surface.parent = self->frame->a_items;
     item_a->surface.parentx = self->area.x;
@@ -410,7 +432,7 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                   ob_rr_theme->a_menu_text_normal);
         sub = self->entry->data.submenu.submenu;
         text_a->texture[0].data.text.string = sub ? sub->title : "";
-        if (sub->shortcut && (self->frame->menu->show_all_shortcuts ||
+        if (sub && sub->shortcut && (self->frame->menu->show_all_shortcuts ||
                               sub->shortcut_always_show ||
                               sub->shortcut_position > 0))
         {
@@ -428,11 +450,13 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
         else
             text_a = ob_rr_theme->a_menu_text_normal;
         break;
+    default:
+        g_assert_not_reached();
     }
 
     switch (self->entry->type) {
     case OB_MENU_ENTRY_TYPE_NORMAL:
-        XMoveResizeWindow(ob_display, self->text,
+        XMoveResizeWindow(obt_display, self->text,
                           self->frame->text_x, PADDING,
                           self->frame->text_w,
                           ITEM_HEIGHT - 2*PADDING);
@@ -443,7 +467,7 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                 ITEM_HEIGHT - 2*PADDING);
         break;
     case OB_MENU_ENTRY_TYPE_SUBMENU:
-        XMoveResizeWindow(ob_display, self->text,
+        XMoveResizeWindow(obt_display, self->text,
                           self->frame->text_x, PADDING,
                           self->frame->text_w - ITEM_HEIGHT,
                           ITEM_HEIGHT - 2*PADDING);
@@ -456,7 +480,7 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
     case OB_MENU_ENTRY_TYPE_SEPARATOR:
         if (self->entry->data.separator.label != NULL) {
             /* labeled separator */
-            XMoveResizeWindow(ob_display, self->text,
+            XMoveResizeWindow(obt_display, self->text,
                               ob_rr_theme->paddingx, ob_rr_theme->paddingy,
                               self->area.width - 2*ob_rr_theme->paddingx,
                               ob_rr_theme->menu_title_height -
@@ -472,7 +496,7 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
             gint i;
 
             /* unlabeled separator */
-            XMoveResizeWindow(ob_display, self->text, 0, 0,
+            XMoveResizeWindow(obt_display, self->text, 0, 0,
                               self->area.width,
                               ob_rr_theme->menu_sep_width +
                               2*ob_rr_theme->menu_sep_paddingy);
@@ -496,14 +520,17 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                     2*ob_rr_theme->menu_sep_paddingy);
         }
         break;
+    default:
+        g_assert_not_reached();
     }
 
-    if (self->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
+    if (((self->entry->type == OB_MENU_ENTRY_TYPE_NORMAL) ||
+         (self->entry->type == OB_MENU_ENTRY_TYPE_SUBMENU)) &&
         self->entry->data.normal.icon)
     {
         RrAppearance *clear;
 
-        XMoveResizeWindow(ob_display, self->icon,
+        XMoveResizeWindow(obt_display, self->icon,
                           PADDING, frame->item_margin.top,
                           ITEM_HEIGHT - frame->item_margin.top
                           - frame->item_margin.bottom,
@@ -525,14 +552,14 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                 - frame->item_margin.bottom,
                 ITEM_HEIGHT - frame->item_margin.top
                 - frame->item_margin.bottom);
-        XMapWindow(ob_display, self->icon);
+        XMapWindow(obt_display, self->icon);
     } else if (self->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
                self->entry->data.normal.mask)
     {
         RrColor *c;
         RrAppearance *clear;
 
-        XMoveResizeWindow(ob_display, self->icon,
+        XMoveResizeWindow(obt_display, self->icon,
                           PADDING, frame->item_margin.top,
                           ITEM_HEIGHT - frame->item_margin.top
                           - frame->item_margin.bottom,
@@ -565,13 +592,13 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                 - frame->item_margin.bottom,
                 ITEM_HEIGHT - frame->item_margin.top
                 - frame->item_margin.bottom);
-        XMapWindow(ob_display, self->icon);
+        XMapWindow(obt_display, self->icon);
     } else
-        XUnmapWindow(ob_display, self->icon);
+        XUnmapWindow(obt_display, self->icon);
 
     if (self->entry->type == OB_MENU_ENTRY_TYPE_SUBMENU) {
         RrAppearance *bullet_a;
-        XMoveResizeWindow(ob_display, self->bullet,
+        XMoveResizeWindow(obt_display, self->bullet,
                           self->frame->text_x + self->frame->text_w -
                           ITEM_HEIGHT + PADDING, PADDING,
                           ITEM_HEIGHT - 2*PADDING,
@@ -586,11 +613,11 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
         RrPaint(bullet_a, self->bullet,
                 ITEM_HEIGHT - 2*PADDING,
                 ITEM_HEIGHT - 2*PADDING);
-        XMapWindow(ob_display, self->bullet);
+        XMapWindow(obt_display, self->bullet);
     } else
-        XUnmapWindow(ob_display, self->bullet);
+        XUnmapWindow(obt_display, self->bullet);
 
-    XFlush(ob_display);
+    XFlush(obt_display);
 }
 
 /*! this code is taken from the menu_frame_render. if that changes, this won't
@@ -654,6 +681,7 @@ void menu_frame_render(ObMenuFrame *self)
     if (self->entries) {
         gint l, t, r, b;
 
+        e = self->entries->data;
         ob_rr_theme->a_menu_text_normal->texture[0].data.text.string = "";
         tw = RrMinWidth(ob_rr_theme->a_menu_text_normal);
         tw += 2*PADDING;
@@ -708,10 +736,10 @@ void menu_frame_render(ObMenuFrame *self)
         }
 
         RECT_SET_POINT(e->area, 0, h+e->border);
-        XMoveWindow(ob_display, e->window,
+        XMoveWindow(obt_display, e->window,
                     e->area.x-e->border, e->area.y-e->border);
-        XSetWindowBorderWidth(ob_display, e->window, e->border);
-        XSetWindowBorder(ob_display, e->window,
+        XSetWindowBorderWidth(obt_display, e->window, e->border);
+        XSetWindowBorder(obt_display, e->window,
                          RrColorPixel(ob_rr_theme->menu_border_color));
 
         text_a = (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
@@ -763,6 +791,8 @@ void menu_frame_render(ObMenuFrame *self)
                     2*ob_rr_theme->menu_sep_paddingy - 2*PADDING;
             }
             break;
+        default:
+            g_assert_not_reached();
         }
         tw += 2*PADDING;
         th += 2*PADDING;
@@ -793,7 +823,7 @@ void menu_frame_render(ObMenuFrame *self)
     if (!w) w = 10;
     if (!h) h = 3;
 
-    XResizeWindow(ob_display, self->window, w, h);
+    XResizeWindow(obt_display, self->window, w, h);
 
     self->inner_w = w;
 
@@ -807,13 +837,13 @@ void menu_frame_render(ObMenuFrame *self)
 
     RECT_SET_SIZE(self->area, w, h);
 
-    XFlush(ob_display);
+    XFlush(obt_display);
 }
 
 static void menu_frame_update(ObMenuFrame *self)
 {
     GList *mit, *fit;
-    Rect *a;
+    const Rect *a;
     gint h;
 
     menu_pipe_execute(self->menu);
@@ -911,8 +941,6 @@ static void menu_frame_update(ObMenuFrame *self)
         }
     }
 
-    g_free(a);
-
     menu_frame_render(self);
 }
 
@@ -977,7 +1005,7 @@ gboolean menu_frame_show_topmenu(ObMenuFrame *self, gint x, gint y,
 
     menu_frame_move(self, x, y);
 
-    XMapWindow(ob_display, self->window);
+    XMapWindow(obt_display, self->window);
 
     if (screen_pointer_pos(&px, &py)) {
         ObMenuEntryFrame *e = menu_entry_frame_under(px, py);
@@ -993,8 +1021,8 @@ gboolean menu_frame_show_topmenu(ObMenuFrame *self, gint x, gint y,
 */
 static void remove_submenu_hide_timeout(ObMenuFrame *child)
 {
-    ob_main_loop_timeout_remove_data(ob_main_loop, submenu_hide_timeout,
-                                     child, FALSE);
+    if (submenu_hide_timer) g_source_remove(submenu_hide_timer);
+    submenu_hide_timer = 0;
 }
 
 gboolean menu_frame_show_submenu(ObMenuFrame *self, ObMenuFrame *parent,
@@ -1032,7 +1060,7 @@ gboolean menu_frame_show_submenu(ObMenuFrame *self, ObMenuFrame *parent,
     }
     menu_frame_move(self, x + dx, y + dy);
 
-    XMapWindow(ob_display, self->window);
+    XMapWindow(obt_display, self->window);
 
     if (screen_pointer_pos(&px, &py)) {
         ObMenuEntryFrame *e = menu_entry_frame_under(px, py);
@@ -1045,14 +1073,15 @@ gboolean menu_frame_show_submenu(ObMenuFrame *self, ObMenuFrame *parent,
 
 static void menu_frame_hide(ObMenuFrame *self)
 {
+    ObMenu *const menu = self->menu;
     GList *it = g_list_find(menu_frame_visible, self);
     gulong ignore_start;
 
     if (!it)
         return;
 
-    if (self->menu->hide_func)
-        self->menu->hide_func(self, self->menu->data);
+    if (menu->hide_func)
+        menu->hide_func(self, menu->data);
 
     if (self->child)
         menu_frame_hide(self->child);
@@ -1075,10 +1104,13 @@ static void menu_frame_hide(ObMenuFrame *self)
     }
 
     ignore_start = event_start_ignore_all_enters();
-    XUnmapWindow(ob_display, self->window);
+    XUnmapWindow(obt_display, self->window);
     event_end_ignore_all_enters(ignore_start);
 
     menu_frame_free(self);
+
+    if (menu->cleanup_func)
+        menu->cleanup_func(menu, menu->data);
 }
 
 void menu_frame_hide_all(void)
@@ -1087,7 +1119,8 @@ void menu_frame_hide_all(void)
 
     if (config_submenu_show_delay) {
         /* remove any submenu open requests */
-        ob_main_loop_timeout_remove(ob_main_loop, submenu_show_timeout);
+        if (submenu_show_timer) g_source_remove(submenu_show_timer);
+        submenu_show_timer = 0;
     }
     if ((it = g_list_last(menu_frame_visible)))
         menu_frame_hide(it->data);
@@ -1168,7 +1201,8 @@ void menu_frame_select(ObMenuFrame *self, ObMenuEntryFrame *entry,
 
     if (config_submenu_show_delay) {
         /* remove any submenu open requests */
-        ob_main_loop_timeout_remove(ob_main_loop, submenu_show_timeout);
+        if (submenu_show_timer) g_source_remove(submenu_show_timer);
+        submenu_show_timer = 0;
     }
 
     self->selected = entry;
@@ -1188,12 +1222,13 @@ void menu_frame_select(ObMenuFrame *self, ObMenuEntryFrame *entry,
                submenu */
             if (immediate || config_submenu_hide_delay == 0)
                 menu_frame_hide(oldchild);
-            else if (config_submenu_hide_delay > 0)
-                ob_main_loop_timeout_add(ob_main_loop,
-                                         config_submenu_hide_delay * 1000,
-                                         submenu_hide_timeout,
-                                         oldchild, g_direct_equal,
-                                         NULL);
+            else if (config_submenu_hide_delay > 0) {
+                if (submenu_hide_timer) g_source_remove(submenu_hide_timer);
+                submenu_hide_timer =
+                    g_timeout_add_full(G_PRIORITY_DEFAULT,
+                                       config_submenu_hide_delay,
+                                       submenu_hide_timeout, oldchild, NULL);
+            }
         }
     }
 
@@ -1205,12 +1240,15 @@ void menu_frame_select(ObMenuFrame *self, ObMenuEntryFrame *entry,
             if (oldchild_entry != self->selected) {
                 if (immediate || config_submenu_hide_delay == 0)
                     menu_entry_frame_show_submenu(self->selected);
-                else if (config_submenu_hide_delay > 0)
-                    ob_main_loop_timeout_add(ob_main_loop,
-                                             config_submenu_show_delay * 1000,
-                                             submenu_show_timeout,
-                                             self->selected, g_direct_equal,
-                                             NULL);
+                else if (config_submenu_hide_delay > 0) {
+                    if (submenu_show_timer)
+                        g_source_remove(submenu_show_timer);
+                    submenu_show_timer =
+                        g_timeout_add_full(G_PRIORITY_DEFAULT,
+                                           config_submenu_show_delay,
+                                           submenu_show_timeout,
+                                           self->selected, NULL);
+                }
             }
             /* hide the grandchildren of this menu. and move the cursor to
                the current menu */
@@ -1250,9 +1288,10 @@ void menu_entry_frame_execute(ObMenuEntryFrame *self, guint state)
         GSList *acts = self->entry->data.normal.actions;
         ObClient *client = self->frame->client;
         ObMenuFrame *frame = self->frame;
+        guint mods = obt_keyboard_only_modmasks(state);
 
         /* release grabs before executing the shit */
-        if (!(state & ControlMask)) {
+        if (!(mods & ControlMask)) {
             event_cancel_all_key_grabs();
             frame = NULL;
         }
