@@ -22,7 +22,8 @@
 
 #include "geom.h"
 #include "window.h"
-#include "render/render.h"
+#include "obrender/render.h"
+#include "obt/keyboard.h"
 
 #include <glib.h>
 
@@ -38,7 +39,7 @@ extern GList *menu_frame_visible;
 struct _ObMenuFrame
 {
     /* stuff to be an ObWindow */
-    Window_InternalType type;
+    ObWindow obwin;
     Window window;
 
     struct _ObMenu *menu;
@@ -81,6 +82,10 @@ struct _ObMenuFrame
                            menu until it has seen a KeyPress.  this is to
                            avoid having the keybinding used to show the menu
                            end up running something inside the menu */
+    guint press_keycode; /* the KeyCode that was used in the last KeyPress */
+    gboolean press_doexec; /* if the upcoming KeyRelease should be used to
+                              execute the menu item that was selected by the
+                              KeyPress */
 };
 
 struct _ObMenuEntryFrame
@@ -109,6 +114,8 @@ ObMenuFrame* menu_frame_new(struct _ObMenu *menu,
                             struct _ObClient *client);
 void menu_frame_free(ObMenuFrame *self);
 
+ObtIC* menu_frame_ic(ObMenuFrame *self);
+
 void menu_frame_move(ObMenuFrame *self, gint x, gint y);
 void menu_frame_move_on_screen(ObMenuFrame *self, gint x, gint y,
                                gint *dx, gint *dy);
