@@ -20,27 +20,29 @@
 #include "grab.h"
 #include "framerender.h"
 #include "screen.h"
-#include "prop.h"
 #include "client.h"
+#include "focus.h"
 #include "frame.h"
 #include "openbox.h"
 #include "resist.h"
-#include "mainloop.h"
-#include "modkeys.h"
 #include "popup.h"
 #include "moveresize.h"
 #include "config.h"
 #include "event.h"
 #include "debug.h"
-#include "extensions.h"
-#include "render/render.h"
-#include "render/theme.h"
+#include "obrender/render.h"
+#include "obrender/theme.h"
+#include "obt/display.h"
+#include "obt/xqueue.h"
+#include "obt/prop.h"
+#include "obt/keyboard.h"
 
 #include <X11/Xlib.h>
 #include <glib.h>
 
 /* how far windows move and resize with the keyboard arrows */
 #define KEY_DIST 8
+#define SYNC_TIMEOUTS 4
 
 gboolean moveresize_in_progress = FALSE;
 ObClient *moveresize_client = NULL;
@@ -50,15 +52,21 @@ XSyncAlarm moveresize_alarm = None;
 
 static gboolean moving = FALSE; /* TRUE - moving, FALSE - resizing */
 
+/* starting geometry for the window being moved/resized, so it can be
+   restored */
 static gint start_x, start_y, start_cx, start_cy, start_cw, start_ch;
+static gboolean was_max_horz, was_max_vert;
+static Rect pre_max_area;
 static gint cur_x, cur_y, cur_w, cur_h;
 static guint button;
 static guint32 corner;
 static ObDirection edge_warp_dir = -1;
 static gboolean edge_warp_odd = FALSE;
+static guint edge_warp_timer = 0;
 static ObDirection key_resize_edge = -1;
+static guint waiting_for_sync;
 #ifdef SYNC
-static gboolean waiting_for_sync;
+static guint sync_timer = 0;
 #endif
 
 static ObPopup *popup = NULL;
@@ -113,7 +121,7 @@ static void popup_coords(ObClient *c, const gchar *format, gint a, gint b)
                        c->frame->area.x + c->frame->area.width / 2,
                        c->frame->area.y + c->frame->area.height / 2);
     else /* Fixed */ {
-        Rect *area = screen_physical_area_active();
+        const Rect *area = screen_physical_area_active();
         gint gravity, x, y;
 
         x = config_resize_popup_fixed.x.pos;
@@ -158,8 +166,6 @@ static void popup_coords(ObClient *c, const gchar *format, gint a, gint b)
         }
 
         popup_position(popup, gravity, x, y);
-
-        g_free(area);
     }
     popup_show(popup, text);
     g_free(text);
@@ -168,8 +174,8 @@ static void popup_coords(ObClient *c, const gchar *format, gint a, gint b)
 void moveresize_start(ObClient *c, gint x, gint y, guint b, guint32 cnr)
 {
     ObCursor cur;
-    gboolean mv = (cnr == prop_atoms.net_wm_moveresize_move ||
-                   cnr == prop_atoms.net_wm_moveresize_move_keyboard);
+    gboolean mv = (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_MOVE) ||
+                   cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_MOVE_KEYBOARD));
     gint up = 1;
     gint left = 1;
 
@@ -179,32 +185,37 @@ void moveresize_start(ObClient *c, gint x, gint y, guint b, guint32 cnr)
           (c->functions & OB_CLIENT_FUNC_RESIZE)))
         return;
 
-    if (cnr == prop_atoms.net_wm_moveresize_size_topleft) {
+    if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOPLEFT)) {
         cur = OB_CURSOR_NORTHWEST;
         up = left = -1;
-    } else if (cnr == prop_atoms.net_wm_moveresize_size_top) {
+    }
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOP)) {
         cur = OB_CURSOR_NORTH;
         up = -1;
-    } else if (cnr == prop_atoms.net_wm_moveresize_size_topright) {
+    }
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOPRIGHT)) {
         cur = OB_CURSOR_NORTHEAST;
         up = -1;
-    } else if (cnr == prop_atoms.net_wm_moveresize_size_right)
+    }
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_RIGHT))
         cur = OB_CURSOR_EAST;
-    else if (cnr == prop_atoms.net_wm_moveresize_size_bottomright)
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT))
         cur = OB_CURSOR_SOUTHEAST;
-    else if (cnr == prop_atoms.net_wm_moveresize_size_bottom)
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_BOTTOM))
         cur = OB_CURSOR_SOUTH;
-    else if (cnr == prop_atoms.net_wm_moveresize_size_bottomleft) {
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT)) {
         cur = OB_CURSOR_SOUTHWEST;
         left = -1;
-    } else if (cnr == prop_atoms.net_wm_moveresize_size_left) {
+    }
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_LEFT)) {
         cur = OB_CURSOR_WEST;
         left = -1;
-    } else if (cnr == prop_atoms.net_wm_moveresize_size_keyboard)
+    }
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_KEYBOARD))
         cur = OB_CURSOR_SOUTHEAST;
-    else if (cnr == prop_atoms.net_wm_moveresize_move)
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_MOVE))
         cur = OB_CURSOR_MOVE;
-    else if (cnr == prop_atoms.net_wm_moveresize_move_keyboard)
+    else if (cnr == OBT_PROP_ATOM(NET_WM_MOVERESIZE_MOVE_KEYBOARD))
         cur = OB_CURSOR_MOVE;
     else
         g_assert_not_reached();
@@ -235,6 +246,9 @@ void moveresize_start(ObClient *c, gint x, gint y, guint b, guint32 cnr)
     button = b;
     key_resize_edge = -1;
 
+    /* default to not putting max back on cancel */
+    was_max_horz = was_max_vert = FALSE;
+
     /*
       have to change start_cx and start_cy if going to do this..
     if (corner == prop_atoms.net_wm_moveresize_move_keyboard ||
@@ -249,9 +263,10 @@ void moveresize_start(ObClient *c, gint x, gint y, guint b, guint32 cnr)
     cur_h = start_ch;
 
     moveresize_in_progress = TRUE;
+    waiting_for_sync = 0;
 
 #ifdef SYNC
-    if (config_resize_redraw && !moving && extensions_sync &&
+    if (config_resize_redraw && !moving && obt_display_extension_sync &&
         moveresize_client->sync_request && moveresize_client->sync_counter &&
         !moveresize_client->not_responding)
     {
@@ -263,7 +278,7 @@ void moveresize_start(ObClient *c, gint x, gint y, guint b, guint32 cnr)
 
         /* set the counter to an initial value */
         XSyncIntToValue(&val, 0);
-        XSyncSetCounter(ob_display, moveresize_client->sync_counter, val);
+        XSyncSetCounter(obt_display, moveresize_client->sync_counter, val);
 
         /* this will be incremented when we tell the client what we're
            looking for */
@@ -279,7 +294,7 @@ void moveresize_start(ObClient *c, gint x, gint y, guint b, guint32 cnr)
         aa.trigger.test_type = XSyncPositiveTransition;
         aa.events = True;
         XSyncIntToValue(&aa.delta, 1);
-        moveresize_alarm = XSyncCreateAlarm(ob_display,
+        moveresize_alarm = XSyncCreateAlarm(obt_display,
                                             XSyncCACounter |
                                             XSyncCAValue |
                                             XSyncCAValueType |
@@ -287,8 +302,6 @@ void moveresize_start(ObClient *c, gint x, gint y, guint b, guint32 cnr)
                                             XSyncCADelta |
                                             XSyncCAEvents,
                                             &aa);
-
-        waiting_for_sync = FALSE;
     }
 #endif
 }
@@ -304,11 +317,12 @@ void moveresize_end(gboolean cancel)
 #ifdef SYNC
         /* turn off the alarm */
         if (moveresize_alarm != None) {
-            XSyncDestroyAlarm(ob_display, moveresize_alarm);
+            XSyncDestroyAlarm(obt_display, moveresize_alarm);
             moveresize_alarm = None;
         }
 
-        ob_main_loop_timeout_remove(ob_main_loop, sync_timeout_func);
+        if (sync_timer) g_source_remove(sync_timer);
+        sync_timer = 0;
 #endif
     }
 
@@ -323,6 +337,28 @@ void moveresize_end(gboolean cancel)
                      (cancel ? start_ch : cur_h),
                      TRUE, TRUE, FALSE);
 
+    /* restore the client's maximized state. do this after putting the window
+       back in its original spot to minimize visible flicker */
+    if (cancel && (was_max_horz || was_max_vert)) {
+        const gboolean h = moveresize_client->max_horz;
+        const gboolean v = moveresize_client->max_vert;
+
+        client_maximize(moveresize_client, TRUE,
+                        was_max_horz && was_max_vert ? 0 :
+                        (was_max_horz ? 1 : 2));
+
+        /* replace the premax values with the ones we had saved if
+           the client doesn't have any already set */
+        if (was_max_horz && !h) {
+            moveresize_client->pre_max_area.x = pre_max_area.x;
+            moveresize_client->pre_max_area.width = pre_max_area.width;
+        }
+        if (was_max_vert && !v) {
+            moveresize_client->pre_max_area.y = pre_max_area.y;
+            moveresize_client->pre_max_area.height = pre_max_area.height;
+        }
+    }
+
     /* dont edge warp after its ended */
     cancel_edge_warp();
 
@@ -352,19 +388,27 @@ static void do_resize(void)
 {
     gint x, y, w, h, lw, lh;
 
-    /* see if it is actually going to resize */
-    x = 0;
-    y = 0;
+    /* see if it is actually going to resize
+       USE cur_x AND cur_y HERE !  Otherwise the try_configure won't know
+       what struts to use !!
+     */
+    x = cur_x;
+    y = cur_y;
     w = cur_w;
     h = cur_h;
     client_try_configure(moveresize_client, &x, &y, &w, &h,
                          &lw, &lh, TRUE);
     if (!(w == moveresize_client->area.width &&
-          h == moveresize_client->area.height))
+          h == moveresize_client->area.height) &&
+        /* if waiting_for_sync == 0, then we aren't waiting.
+           if it is > SYNC_TIMEOUTS, then we have timed out
+           that many times already, so forget about waiting more */
+        (waiting_for_sync == 0 || waiting_for_sync > SYNC_TIMEOUTS))
     {
-
 #ifdef SYNC
-        if (config_resize_redraw && extensions_sync &&
+        if (config_resize_redraw && obt_display_extension_sync &&
+            /* don't send another sync when one is pending */
+            waiting_for_sync == 0 &&
             moveresize_client->sync_request &&
             moveresize_client->sync_counter &&
             !moveresize_client->not_responding)
@@ -372,38 +416,35 @@ static void do_resize(void)
             XEvent ce;
             XSyncValue val;
 
-            /* are we already waiting for the sync counter to catch up? */
-            if (waiting_for_sync)
-                return;
-
             /* increment the value we're waiting for */
             ++moveresize_client->sync_counter_value;
             XSyncIntToValue(&val, moveresize_client->sync_counter_value);
 
             /* tell the client what we're waiting for */
             ce.xclient.type = ClientMessage;
-            ce.xclient.message_type = prop_atoms.wm_protocols;
-            ce.xclient.display = ob_display;
+            ce.xclient.message_type = OBT_PROP_ATOM(WM_PROTOCOLS);
+            ce.xclient.display = obt_display;
             ce.xclient.window = moveresize_client->window;
             ce.xclient.format = 32;
-            ce.xclient.data.l[0] = prop_atoms.net_wm_sync_request;
-            ce.xclient.data.l[1] = event_curtime;
+            ce.xclient.data.l[0] = OBT_PROP_ATOM(NET_WM_SYNC_REQUEST);
+            ce.xclient.data.l[1] = event_time();
             ce.xclient.data.l[2] = XSyncValueLow32(val);
             ce.xclient.data.l[3] = XSyncValueHigh32(val);
             ce.xclient.data.l[4] = 0l;
-            XSendEvent(ob_display, moveresize_client->window, FALSE,
+            XSendEvent(obt_display, moveresize_client->window, FALSE,
                        NoEventMask, &ce);
 
-            waiting_for_sync = TRUE;
+            waiting_for_sync = 1;
 
-            ob_main_loop_timeout_remove(ob_main_loop, sync_timeout_func);
-            ob_main_loop_timeout_add(ob_main_loop, G_USEC_PER_SEC * 2,
-                                     sync_timeout_func, NULL, NULL, NULL);
+            if (sync_timer) g_source_remove(sync_timer);
+            sync_timer = g_timeout_add(2000, sync_timeout_func, NULL);
         }
 #endif
 
+        /* force a ConfigureNotify, it is part of the spec for SYNC resizing
+           and MUST follow the sync counter notification */
         client_configure(moveresize_client, cur_x, cur_y, cur_w, cur_h,
-                TRUE, FALSE, FALSE);
+                         TRUE, FALSE, TRUE);
     }
 
     /* this would be better with a fixed width font ... XXX can do it better
@@ -418,10 +459,15 @@ static void do_resize(void)
 #ifdef SYNC
 static gboolean sync_timeout_func(gpointer data)
 {
-    waiting_for_sync = FALSE; /* we timed out waiting for our sync... */
+    ++waiting_for_sync; /* we timed out waiting for our sync... */
     do_resize(); /* ...so let any pending resizes through */
 
-    return FALSE; /* don't repeat */
+    if (waiting_for_sync > SYNC_TIMEOUTS) {
+        sync_timer = 0;
+        return FALSE; /* don't repeat */
+    }
+    else
+        return TRUE; /* keep waiting */
 }
 #endif
 
@@ -527,6 +573,34 @@ static void calc_resize(gboolean keyboard, gint keydist, gint *dw, gint *dh,
     *dh = nh - oh;
 }
 
+static void edge_warp_move_ptr(void)
+{
+    gint x, y;
+    const Rect* a;
+
+    screen_pointer_pos(&x, &y);
+    a = screen_physical_area_all_monitors();
+
+    switch (edge_warp_dir) {
+	case OB_DIRECTION_NORTH:
+	    y = a->height - 1;
+	    break;
+	case OB_DIRECTION_EAST:
+	    x = a->x;
+	    break;
+	case OB_DIRECTION_SOUTH:
+	    y = a->y;
+	    break;
+	case OB_DIRECTION_WEST:
+	    x = a->width - 1;
+	    break;
+	default:
+        g_assert_not_reached();
+    }
+
+    XWarpPointer(obt_display, 0, obt_root(ob_screen), 0, 0, 0, 0, x, y);
+}
+
 static gboolean edge_warp_delay_func(gpointer data)
 {
     guint d;
@@ -535,7 +609,10 @@ static gboolean edge_warp_delay_func(gpointer data)
        after that */
     if (edge_warp_odd) {
         d = screen_find_desktop(screen_desktop, edge_warp_dir, TRUE, FALSE);
-        if (d != screen_desktop) screen_set_desktop(d, TRUE);
+        if (d != screen_desktop) {
+            if (config_mouse_screenedgewarp) edge_warp_move_ptr();
+            screen_set_desktop(d, TRUE);
+        }
     }
     edge_warp_odd = !edge_warp_odd;
 
@@ -552,7 +629,7 @@ static void do_edge_warp(gint x, gint y)
     dir = -1;
 
     for (i = 0; i < screen_num_monitors; ++i) {
-        Rect *a = screen_physical_area_monitor(i);
+        const Rect *a = screen_physical_area_monitor(i);
         if (x == RECT_LEFT(*a)) dir = OB_DIRECTION_WEST;
         if (x == RECT_RIGHT(*a)) dir = OB_DIRECTION_EAST;
         if (y == RECT_TOP(*a)) dir = OB_DIRECTION_NORTH;
@@ -569,17 +646,14 @@ static void do_edge_warp(gint x, gint y)
         {
             dir = -1;
         }
-        g_free(a);
     }
 
     if (dir != edge_warp_dir) {
         cancel_edge_warp();
         if (dir != (ObDirection)-1) {
             edge_warp_odd = TRUE; /* switch on the first timeout */
-            ob_main_loop_timeout_add(ob_main_loop,
-                                     config_mouse_screenedgetime * 1000,
-                                     edge_warp_delay_func,
-                                     NULL, NULL, NULL);
+            edge_warp_timer = g_timeout_add(config_mouse_screenedgetime,
+                                            edge_warp_delay_func, NULL);
         }
         edge_warp_dir = dir;
     }
@@ -587,27 +661,29 @@ static void do_edge_warp(gint x, gint y)
 
 static void cancel_edge_warp(void)
 {
-    ob_main_loop_timeout_remove(ob_main_loop, edge_warp_delay_func);
+    if (edge_warp_timer) g_source_remove(edge_warp_timer);
+    edge_warp_timer = 0;
 }
 
-static void move_with_keys(gint keycode, gint state)
+static void move_with_keys(KeySym sym, guint state)
 {
     gint dx = 0, dy = 0, ox = cur_x, oy = cur_y;
     gint opx, px, opy, py;
     gint dist = 0;
 
     /* shift means jump to edge */
-    if (state & modkeys_key_to_mask(OB_MODKEY_KEY_SHIFT)) {
+    if (state & obt_keyboard_modkey_to_modmask(OBT_KEYBOARD_MODKEY_SHIFT))
+    {
         gint x, y;
         ObDirection dir;
 
-        if (ob_keycode_match(keycode, OB_KEY_RIGHT))
+        if (sym == XK_Right)
             dir = OB_DIRECTION_EAST;
-        else if (ob_keycode_match(keycode, OB_KEY_LEFT))
+        else if (sym == XK_Left)
             dir = OB_DIRECTION_WEST;
-        else if (ob_keycode_match(keycode, OB_KEY_DOWN))
+        else if (sym == XK_Down)
             dir = OB_DIRECTION_SOUTH;
-        else /* if (ob_keycode_match(keycode, OB_KEY_UP)) */
+        else /* sym == XK_Up */
             dir = OB_DIRECTION_NORTH;
 
         client_find_move_directional(moveresize_client, dir, &x, &y);
@@ -615,28 +691,32 @@ static void move_with_keys(gint keycode, gint state)
         dy = y - moveresize_client->area.y;
     } else {
         /* control means fine grained */
-        if (state & modkeys_key_to_mask(OB_MODKEY_KEY_CONTROL))
+        if (state &
+            obt_keyboard_modkey_to_modmask(OBT_KEYBOARD_MODKEY_CONTROL))
+        {
             dist = 1;
+        }
         else
             dist = KEY_DIST;
 
-        if (ob_keycode_match(keycode, OB_KEY_RIGHT))
+        if (sym == XK_Right)
             dx = dist;
-        else if (ob_keycode_match(keycode, OB_KEY_LEFT))
+        else if (sym == XK_Left)
             dx = -dist;
-        else if (ob_keycode_match(keycode, OB_KEY_DOWN))
+        else if (sym == XK_Down)
             dy = dist;
-        else /* if (ob_keycode_match(keycode, OB_KEY_UP)) */
+        else /* if (sym == XK_Up) */
             dy = -dist;
     }
 
     screen_pointer_pos(&opx, &opy);
-    XWarpPointer(ob_display, None, None, 0, 0, 0, 0, dx, dy);
+    XWarpPointer(obt_display, None, None, 0, 0, 0, 0, dx, dy);
     /* steal the motion events this causes */
-    XSync(ob_display, FALSE);
+    XSync(obt_display, FALSE);
     {
         XEvent ce;
-        while (XCheckTypedEvent(ob_display, MotionNotify, &ce));
+        while (xqueue_remove_local(&ce, xqueue_match_type,
+                                   GINT_TO_POINTER(MotionNotify)));
     }
     screen_pointer_pos(&px, &py);
 
@@ -652,14 +732,14 @@ static void move_with_keys(gint keycode, gint state)
     start_y += (py - opy) - (cur_y - oy);
 }
 
-static void resize_with_keys(gint keycode, gint state)
+static void resize_with_keys(KeySym sym, guint state)
 {
     gint dw = 0, dh = 0, pdx = 0, pdy = 0, opx, opy, px, py;
     gint resist = 0;
     ObDirection dir;
 
     /* pick the edge if it needs to move */
-    if (ob_keycode_match(keycode, OB_KEY_RIGHT)) {
+    if (sym == XK_Right) {
         dir = OB_DIRECTION_EAST;
         if (key_resize_edge != OB_DIRECTION_WEST &&
             key_resize_edge != OB_DIRECTION_EAST)
@@ -667,8 +747,7 @@ static void resize_with_keys(gint keycode, gint state)
             key_resize_edge = OB_DIRECTION_EAST;
             return;
         }
-    }
-    if (ob_keycode_match(keycode, OB_KEY_LEFT)) {
+    } else if (sym == XK_Left) {
         dir = OB_DIRECTION_WEST;
         if (key_resize_edge != OB_DIRECTION_WEST &&
             key_resize_edge != OB_DIRECTION_EAST)
@@ -676,8 +755,7 @@ static void resize_with_keys(gint keycode, gint state)
             key_resize_edge = OB_DIRECTION_WEST;
             return;
         }
-    }
-    if (ob_keycode_match(keycode, OB_KEY_UP)) {
+    } else if (sym == XK_Up) {
         dir = OB_DIRECTION_NORTH;
         if (key_resize_edge != OB_DIRECTION_NORTH &&
             key_resize_edge != OB_DIRECTION_SOUTH)
@@ -685,8 +763,7 @@ static void resize_with_keys(gint keycode, gint state)
             key_resize_edge = OB_DIRECTION_NORTH;
             return;
         }
-    }
-    if (ob_keycode_match(keycode, OB_KEY_DOWN)) {
+    } else /* if (sym == XK_Down) */ {
         dir = OB_DIRECTION_SOUTH;
         if (key_resize_edge != OB_DIRECTION_NORTH &&
             key_resize_edge != OB_DIRECTION_SOUTH)
@@ -697,16 +774,17 @@ static void resize_with_keys(gint keycode, gint state)
     }
 
     /* shift means jump to edge */
-    if (state & modkeys_key_to_mask(OB_MODKEY_KEY_SHIFT)) {
+    if (state & obt_keyboard_modkey_to_modmask(OBT_KEYBOARD_MODKEY_SHIFT))
+    {
         gint x, y, w, h;
 
-        if (ob_keycode_match(keycode, OB_KEY_RIGHT))
+        if (sym == XK_Right)
             dir = OB_DIRECTION_EAST;
-        else if (ob_keycode_match(keycode, OB_KEY_LEFT))
+        else if (sym == XK_Left)
             dir = OB_DIRECTION_WEST;
-        else if (ob_keycode_match(keycode, OB_KEY_DOWN))
+        else if (sym == XK_Down)
             dir = OB_DIRECTION_SOUTH;
-        else /* if (ob_keycode_match(keycode, OB_KEY_UP)) */
+        else /* if (sym == XK_Up)) */
             dir = OB_DIRECTION_NORTH;
 
         client_find_resize_directional(moveresize_client, key_resize_edge,
@@ -722,7 +800,9 @@ static void resize_with_keys(gint keycode, gint state)
             distw = moveresize_client->size_inc.width;
             resist = 1;
         }
-        else if (state & modkeys_key_to_mask(OB_MODKEY_KEY_CONTROL)) {
+        else if (state &
+                 obt_keyboard_modkey_to_modmask(OBT_KEYBOARD_MODKEY_CONTROL))
+        {
             distw = 1;
             resist = 1;
         }
@@ -734,7 +814,9 @@ static void resize_with_keys(gint keycode, gint state)
             disth = moveresize_client->size_inc.height;
             resist = 1;
         }
-        else if (state & modkeys_key_to_mask(OB_MODKEY_KEY_CONTROL)) {
+        else if (state &
+                 obt_keyboard_modkey_to_modmask(OBT_KEYBOARD_MODKEY_CONTROL))
+        {
             disth = 1;
             resist = 1;
         }
@@ -769,6 +851,33 @@ static void resize_with_keys(gint keycode, gint state)
         }
     }
 
+    if (moveresize_client->max_horz &&
+        (key_resize_edge == OB_DIRECTION_WEST ||
+         key_resize_edge == OB_DIRECTION_EAST))
+    {
+        /* unmax horz */
+        was_max_horz = TRUE;
+        pre_max_area.x = moveresize_client->pre_max_area.x;
+        pre_max_area.width = moveresize_client->pre_max_area.width;
+
+        moveresize_client->pre_max_area.x = cur_x;
+        moveresize_client->pre_max_area.width = cur_w;
+        client_maximize(moveresize_client, FALSE, 1);
+    }
+    else if (moveresize_client->max_vert &&
+             (key_resize_edge == OB_DIRECTION_NORTH ||
+              key_resize_edge == OB_DIRECTION_SOUTH))
+    {
+        /* unmax vert */
+        was_max_vert = TRUE;
+        pre_max_area.y = moveresize_client->pre_max_area.y;
+        pre_max_area.height = moveresize_client->pre_max_area.height;
+
+        moveresize_client->pre_max_area.y = cur_y;
+        moveresize_client->pre_max_area.height = cur_h;
+        client_maximize(moveresize_client, FALSE, 2);
+    }
+
     calc_resize(TRUE, resist, &dw, &dh, dir);
     if (key_resize_edge == OB_DIRECTION_WEST)
         cur_x -= dw;
@@ -788,12 +897,13 @@ static void resize_with_keys(gint keycode, gint state)
         pdy = dh;
 
     screen_pointer_pos(&opx, &opy);
-    XWarpPointer(ob_display, None, None, 0, 0, 0, 0, pdx, pdy);
+    XWarpPointer(obt_display, None, None, 0, 0, 0, 0, pdx, pdy);
     /* steal the motion events this causes */
-    XSync(ob_display, FALSE);
+    XSync(obt_display, FALSE);
     {
         XEvent ce;
-        while (XCheckTypedEvent(ob_display, MotionNotify, &ce));
+        while (xqueue_remove_local(&ce, xqueue_match_type,
+                                   GINT_TO_POINTER(MotionNotify)));
     }
     screen_pointer_pos(&px, &py);
 
@@ -836,47 +946,90 @@ gboolean moveresize_event(XEvent *e)
             gint dw, dh;
             ObDirection dir;
 
-            if (corner == prop_atoms.net_wm_moveresize_size_topleft) {
+            if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOPLEFT)) {
                 dw = -(e->xmotion.x_root - start_x);
                 dh = -(e->xmotion.y_root - start_y);
                 dir = OB_DIRECTION_NORTHWEST;
-            } else if (corner == prop_atoms.net_wm_moveresize_size_top) {
+            } else if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOP)) {
                 dw = 0;
                 dh = -(e->xmotion.y_root - start_y);
                 dir = OB_DIRECTION_NORTH;
-            } else if (corner == prop_atoms.net_wm_moveresize_size_topright) {
+            } else if (corner ==
+                       OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOPRIGHT)) {
                 dw = (e->xmotion.x_root - start_x);
                 dh = -(e->xmotion.y_root - start_y);
                 dir = OB_DIRECTION_NORTHEAST;
-            } else if (corner == prop_atoms.net_wm_moveresize_size_right) {
+            } else if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_RIGHT)) {
                 dw = (e->xmotion.x_root - start_x);
                 dh = 0;
                 dir = OB_DIRECTION_EAST;
             } else if (corner ==
-                       prop_atoms.net_wm_moveresize_size_bottomright) {
+                       OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT)) {
                 dw = (e->xmotion.x_root - start_x);
                 dh = (e->xmotion.y_root - start_y);
                 dir = OB_DIRECTION_SOUTHEAST;
-            } else if (corner == prop_atoms.net_wm_moveresize_size_bottom) {
+            } else if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_BOTTOM))
+            {
                 dw = 0;
                 dh = (e->xmotion.y_root - start_y);
                 dir = OB_DIRECTION_SOUTH;
             } else if (corner ==
-                       prop_atoms.net_wm_moveresize_size_bottomleft) {
+                       OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT)) {
                 dw = -(e->xmotion.x_root - start_x);
                 dh = (e->xmotion.y_root - start_y);
                 dir = OB_DIRECTION_SOUTHWEST;
-            } else if (corner == prop_atoms.net_wm_moveresize_size_left) {
+            } else if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_LEFT)) {
                 dw = -(e->xmotion.x_root - start_x);
                 dh = 0;
                 dir = OB_DIRECTION_WEST;
-            } else if (corner == prop_atoms.net_wm_moveresize_size_keyboard) {
+            } else if (corner ==
+                       OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_KEYBOARD)) {
                 dw = (e->xmotion.x_root - start_x);
                 dh = (e->xmotion.y_root - start_y);
                 dir = OB_DIRECTION_SOUTHEAST;
             } else
                 g_assert_not_reached();
 
+            /* override the client's max state if desired */
+            if (ABS(dw) >= config_resist_edge) {
+                if (moveresize_client->max_horz) {
+                    /* unmax horz */
+                    was_max_horz = TRUE;
+                    pre_max_area.x = moveresize_client->pre_max_area.x;
+                    pre_max_area.width = moveresize_client->pre_max_area.width;
+
+                    moveresize_client->pre_max_area.x = cur_x;
+                    moveresize_client->pre_max_area.width = cur_w;
+                    client_maximize(moveresize_client, FALSE, 1);
+                }
+            }
+            else if (was_max_horz && !moveresize_client->max_horz) {
+                /* remax horz and put the premax back */
+                client_maximize(moveresize_client, TRUE, 1);
+                moveresize_client->pre_max_area.x = pre_max_area.x;
+                moveresize_client->pre_max_area.width = pre_max_area.width;
+            }
+
+            if (ABS(dh) >= config_resist_edge) {
+                if (moveresize_client->max_vert) {
+                    /* unmax vert */
+                    was_max_vert = TRUE;
+                    pre_max_area.y = moveresize_client->pre_max_area.y;
+                    pre_max_area.height =
+                        moveresize_client->pre_max_area.height;
+
+                    moveresize_client->pre_max_area.y = cur_y;
+                    moveresize_client->pre_max_area.height = cur_h;
+                    client_maximize(moveresize_client, FALSE, 2);
+                }
+            }
+            else if (was_max_vert && !moveresize_client->max_vert) {
+                /* remax vert and put the premax back */
+                client_maximize(moveresize_client, TRUE, 2);
+                moveresize_client->pre_max_area.y = pre_max_area.y;
+                moveresize_client->pre_max_area.height = pre_max_area.height;
+            }
+
             dw -= cur_w - start_cw;
             dh -= cur_h - start_ch;
 
@@ -884,15 +1037,15 @@ gboolean moveresize_event(XEvent *e)
             cur_w += dw;
             cur_h += dh;
 
-            if (corner == prop_atoms.net_wm_moveresize_size_topleft ||
-                corner == prop_atoms.net_wm_moveresize_size_left ||
-                corner == prop_atoms.net_wm_moveresize_size_bottomleft)
+            if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOPLEFT) ||
+                corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_LEFT) ||
+                corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT))
             {
                 cur_x -= dw;
             }
-            if (corner == prop_atoms.net_wm_moveresize_size_topleft ||
-                corner == prop_atoms.net_wm_moveresize_size_top ||
-                corner == prop_atoms.net_wm_moveresize_size_topright)
+            if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOPLEFT) ||
+                corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOP) ||
+                corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_TOPRIGHT))
             {
                 cur_y -= dh;
             }
@@ -901,33 +1054,39 @@ gboolean moveresize_event(XEvent *e)
         }
         used = TRUE;
     } else if (e->type == KeyPress) {
-        if (ob_keycode_match(e->xkey.keycode, OB_KEY_ESCAPE)) {
+        KeySym sym = obt_keyboard_keypress_to_keysym(e);
+
+        if (sym == XK_Escape) {
             moveresize_end(TRUE);
             used = TRUE;
-        } else if (ob_keycode_match(e->xkey.keycode, OB_KEY_RETURN)) {
+        } else if (sym == XK_Return || sym == XK_KP_Enter) {
             moveresize_end(FALSE);
             used = TRUE;
-        } else if (ob_keycode_match(e->xkey.keycode, OB_KEY_RIGHT) ||
-                   ob_keycode_match(e->xkey.keycode, OB_KEY_LEFT) ||
-                   ob_keycode_match(e->xkey.keycode, OB_KEY_DOWN) ||
-                   ob_keycode_match(e->xkey.keycode, OB_KEY_UP))
+        } else if (sym == XK_Right || sym == XK_Left ||
+                   sym == XK_Up || sym == XK_Down)
         {
-            if (corner == prop_atoms.net_wm_moveresize_size_keyboard) {
-                resize_with_keys(e->xkey.keycode, e->xkey.state);
+            if (corner == OBT_PROP_ATOM(NET_WM_MOVERESIZE_SIZE_KEYBOARD)) {
+                resize_with_keys(sym, e->xkey.state);
                 used = TRUE;
-            } else if (corner == prop_atoms.net_wm_moveresize_move_keyboard) {
-                move_with_keys(e->xkey.keycode, e->xkey.state);
+            } else if (corner ==
+                       OBT_PROP_ATOM(NET_WM_MOVERESIZE_MOVE_KEYBOARD))
+            {
+                move_with_keys(sym, e->xkey.state);
                 used = TRUE;
             }
         }
     }
 #ifdef SYNC
-    else if (e->type == extensions_sync_event_basep + XSyncAlarmNotify)
+    else if (e->type == obt_display_extension_sync_basep + XSyncAlarmNotify)
     {
-        waiting_for_sync = FALSE; /* we got our sync... */
+        waiting_for_sync = 0; /* we got our sync... */
         do_resize(); /* ...so try resize if there is more change pending */
         used = TRUE;
     }
 #endif
+
+    if (used && moveresize_client == focus_client)
+        event_update_user_time();
+
     return used;
 }
