@@ -21,12 +21,9 @@
 #include "openbox.h"
 #include "session.h"
 #include "dock.h"
-#include "modkeys.h"
 #include "event.h"
 #include "menu.h"
 #include "client.h"
-#include "xerror.h"
-#include "prop.h"
 #include "screen.h"
 #include "actions.h"
 #include "startupnotify.h"
@@ -39,18 +36,21 @@
 #include "framerender.h"
 #include "keyboard.h"
 #include "mouse.h"
-#include "extensions.h"
 #include "menuframe.h"
 #include "grab.h"
 #include "group.h"
 #include "config.h"
 #include "ping.h"
-#include "mainloop.h"
 #include "prompt.h"
 #include "gettext.h"
-#include "parser/parse.h"
-#include "render/render.h"
-#include "render/theme.h"
+#include "obrender/render.h"
+#include "obrender/theme.h"
+#include "obt/display.h"
+#include "obt/xqueue.h"
+#include "obt/signal.h"
+#include "obt/prop.h"
+#include "obt/keyboard.h"
+#include "obt/xml.h"
 
 #ifdef HAVE_FCNTL_H
 #  include <fcntl.h>
@@ -82,14 +82,10 @@
 #include <X11/Xcursor/Xcursor.h>
 #endif
 
-#include <X11/Xlib.h>
-#include <X11/keysym.h>
-
 RrInstance   *ob_rr_inst;
 RrImageCache *ob_rr_icons;
 RrTheme      *ob_rr_theme;
-ObMainLoop   *ob_main_loop;
-Display      *ob_display;
+GMainLoop    *ob_main_loop;
 gint          ob_screen;
 gboolean      ob_replace_wm = FALSE;
 gboolean      ob_sm_use = TRUE;
@@ -97,6 +93,7 @@ gchar        *ob_sm_id = NULL;
 gchar        *ob_sm_save_file = NULL;
 gboolean      ob_sm_restore = TRUE;
 gboolean      ob_debug_xinerama = FALSE;
+const gchar  *ob_locale_msg = NULL;
 
 static ObState   state;
 static gboolean  xsync = FALSE;
@@ -104,25 +101,32 @@ static gboolean  reconfigure = FALSE;
 static gboolean  restart = FALSE;
 static gchar    *restart_path = NULL;
 static Cursor    cursors[OB_NUM_CURSORS];
-static KeyCode  *keys[OB_NUM_KEYS];
 static gint      exitcode = 0;
 static guint     remote_control = 0;
 static gboolean  being_replaced = FALSE;
 static gchar    *config_file = NULL;
+static gchar    *startup_cmd = NULL;
 
 static void signal_handler(gint signal, gpointer data);
 static void remove_args(gint *argc, gchar **argv, gint index, gint num);
 static void parse_env();
 static void parse_args(gint *argc, gchar **argv);
 static Cursor load_cursor(const gchar *name, guint fontval);
+static void run_startup_cmd(void);
 
 gint main(gint argc, gchar **argv)
 {
     gchar *program_name;
 
+    obt_signal_listen();
+
     ob_set_state(OB_STATE_STARTING);
 
+    ob_debug_startup();
+
     /* initialize the locale */
+    if (!(ob_locale_msg = setlocale(LC_MESSAGES, "")))
+        g_message("Couldn't set messages locale category from environment.");
     if (!setlocale(LC_ALL, ""))
         g_message("Couldn't set locale from environment.");
     bindtextdomain(PACKAGE_NAME, LOCALEDIR);
@@ -141,46 +145,39 @@ gint main(gint argc, gchar **argv)
     program_name = g_path_get_basename(argv[0]);
     g_set_prgname(program_name);
 
-    if (!remote_control) {
-        parse_paths_startup();
-
+    if (!remote_control)
         session_startup(argc, argv);
-    }
 
-    ob_display = XOpenDisplay(NULL);
-    if (ob_display == NULL)
+    if (!obt_display_open(NULL))
         ob_exit_with_error(_("Failed to open the display from the DISPLAY environment variable."));
-    if (fcntl(ConnectionNumber(ob_display), F_SETFD, 1) == -1)
-        ob_exit_with_error("Failed to set display as close-on-exec");
 
     if (remote_control) {
-        prop_startup();
-
         /* Send client message telling the OB process to:
          * remote_control = 1 -> reconfigure
          * remote_control = 2 -> restart */
-        PROP_MSG(RootWindow(ob_display, ob_screen),
-                 ob_control, remote_control, 0, 0, 0);
-        XCloseDisplay(ob_display);
+        OBT_PROP_MSG(ob_screen, obt_root(ob_screen),
+                     OB_CONTROL, remote_control, 0, 0, 0, 0);
+        obt_display_close();
         exit(EXIT_SUCCESS);
     }
 
-    ob_main_loop = ob_main_loop_new(ob_display);
+    ob_main_loop = g_main_loop_new(NULL, FALSE);
 
-    /* set up signal handler */
-    ob_main_loop_signal_add(ob_main_loop, SIGUSR1, signal_handler, NULL, NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGUSR2, signal_handler, NULL, NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGTERM, signal_handler, NULL, NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGINT, signal_handler, NULL, NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGHUP, signal_handler, NULL, NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGPIPE, signal_handler, NULL, NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGCHLD, signal_handler, NULL, NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGTTIN, signal_handler, NULL,NULL);
-    ob_main_loop_signal_add(ob_main_loop, SIGTTOU, signal_handler, NULL,NULL);
+    /* set up signal handlers, they are called from the mainloop
+       in the main program's thread */
+    obt_signal_add_callback(SIGUSR1, signal_handler, NULL);
+    obt_signal_add_callback(SIGUSR2, signal_handler, NULL);
+    obt_signal_add_callback(SIGTERM, signal_handler, NULL);
+    obt_signal_add_callback(SIGINT,  signal_handler, NULL);
+    obt_signal_add_callback(SIGHUP,  signal_handler, NULL);
+    obt_signal_add_callback(SIGPIPE, signal_handler, NULL);
+    obt_signal_add_callback(SIGCHLD, signal_handler, NULL);
+    obt_signal_add_callback(SIGTTIN, signal_handler, NULL);
+    obt_signal_add_callback(SIGTTOU, signal_handler, NULL);
 
-    ob_screen = DefaultScreen(ob_display);
+    ob_screen = DefaultScreen(obt_display);
 
-    ob_rr_inst = RrInstanceNew(ob_display, ob_screen);
+    ob_rr_inst = RrInstanceNew(obt_display, ob_screen);
     if (ob_rr_inst == NULL)
         ob_exit_with_error(_("Failed to initialize the obrender library."));
     /* Saving 3 resizes of an RrImage makes a lot of sense for icons, as there
@@ -189,7 +186,7 @@ gint main(gint argc, gchar **argv)
     */
     ob_rr_icons = RrImageCacheNew(3);
 
-    XSynchronize(ob_display, xsync);
+    XSynchronize(obt_display, xsync);
 
     /* check for locale support */
     if (!XSupportsLocale())
@@ -197,12 +194,9 @@ gint main(gint argc, gchar **argv)
     if (!XSetLocaleModifiers(""))
         g_message(_("Cannot set locale modifiers for the X server."));
 
-    /* set our error handler */
-    XSetErrorHandler(xerror_handler);
-
     /* set the DISPLAY environment variable for any lauched children, to the
        display we're using, so they open in the right place. */
-    setenv("DISPLAY", DisplayString(ob_display), TRUE);
+    g_setenv("DISPLAY", DisplayString(obt_display), TRUE);
 
     /* create available cursors */
     cursors[OB_CURSOR_NONE] = None;
@@ -223,35 +217,23 @@ gint main(gint argc, gchar **argv)
     cursors[OB_CURSOR_NORTHWEST] = load_cursor("top_left_corner",
                                                XC_top_left_corner);
 
-    prop_startup(); /* get atoms values for the display */
-    extensions_query_all(); /* find which extensions are present */
-
     if (screen_annex()) { /* it will be ours! */
+
+        /* get a timestamp from after taking over as the WM.  if we use the
+           old timestamp to set focus it can fail when replacing another WM. */
+        event_reset_time();
+
         do {
             ObPrompt *xmlprompt = NULL;
 
-            modkeys_startup(reconfigure);
-
-            /* get the keycodes for keys we use */
-            keys[OB_KEY_RETURN] = modkeys_sym_to_code(XK_Return);
-            keys[OB_KEY_ESCAPE] = modkeys_sym_to_code(XK_Escape);
-            keys[OB_KEY_LEFT] = modkeys_sym_to_code(XK_Left);
-            keys[OB_KEY_RIGHT] = modkeys_sym_to_code(XK_Right);
-            keys[OB_KEY_UP] = modkeys_sym_to_code(XK_Up);
-            keys[OB_KEY_DOWN] = modkeys_sym_to_code(XK_Down);
-            keys[OB_KEY_TAB] = modkeys_sym_to_code(XK_Tab);
-            keys[OB_KEY_SPACE] = modkeys_sym_to_code(XK_space);
-            keys[OB_KEY_HOME] = modkeys_sym_to_code(XK_Home);
-            keys[OB_KEY_END] = modkeys_sym_to_code(XK_End);
+            if (reconfigure) obt_keyboard_reload();
 
             {
-                ObParseInst *i;
-                xmlDocPtr doc;
-                xmlNodePtr node;
+                ObtXmlInst *i;
 
                 /* startup the parsing so everything can register sections
                    of the rc */
-                i = parse_startup();
+                i = obt_xml_instance_new();
 
                 /* register all the available actions */
                 actions_startup(reconfigure);
@@ -259,9 +241,13 @@ gint main(gint argc, gchar **argv)
                 config_startup(i);
 
                 /* parse/load user options */
-                if (parse_load_rc(config_file, &doc, &node)) {
-                    parse_tree(i, doc, node->xmlChildrenNode);
-                    parse_close(doc);
+                if ((config_file &&
+                     obt_xml_load_file(i, config_file, "openbox_config")) ||
+                    obt_xml_load_config_file(i, "openbox", "rc.xml",
+                                             "openbox_config"))
+                {
+                    obt_xml_tree_from_root(i);
+                    obt_xml_close(i);
                 }
                 else {
                     g_message(_("Unable to find a valid config file, using some simple defaults"));
@@ -272,16 +258,14 @@ gint main(gint argc, gchar **argv)
                     gchar *p = g_filename_to_utf8(config_file, -1,
                                                   NULL, NULL, NULL);
                     if (p)
-                        PROP_SETS(RootWindow(ob_display, ob_screen),
-                                  ob_config_file, p);
+                        OBT_PROP_SETS(obt_root(ob_screen), OB_CONFIG_FILE, p);
                     g_free(p);
                 }
                 else
-                    PROP_ERASE(RootWindow(ob_display, ob_screen),
-                               ob_config_file);
+                    OBT_PROP_ERASE(obt_root(ob_screen), OB_CONFIG_FILE);
 
                 /* we're done with parsing now, kill it */
-                parse_shutdown(i);
+                obt_xml_instance_unref(i);
             }
 
             /* load the theme specified in the rc file */
@@ -292,7 +276,8 @@ gint main(gint argc, gchar **argv)
                                         config_font_inactivewindow,
                                         config_font_menutitle,
                                         config_font_menuitem,
-                                        config_font_osd)))
+                                        config_font_activeosd,
+                                        config_font_inactiveosd)))
                 {
                     RrThemeFree(ob_rr_theme);
                     ob_rr_theme = theme;
@@ -300,8 +285,8 @@ gint main(gint argc, gchar **argv)
                 if (ob_rr_theme == NULL)
                     ob_exit_with_error(_("Unable to load a theme."));
 
-                PROP_SETS(RootWindow(ob_display, ob_screen),
-                          ob_theme, ob_rr_theme->name);
+                OBT_PROP_SETS(obt_root(ob_screen), OB_THEME,
+                              ob_rr_theme->name);
             }
 
             if (reconfigure) {
@@ -335,19 +320,21 @@ gint main(gint argc, gchar **argv)
             menu_startup(reconfigure);
             prompt_startup(reconfigure);
 
+            /* do this after everything is started so no events will get
+               missed */
+            xqueue_listen();
+
             if (!reconfigure) {
                 guint32 xid;
                 ObWindow *w;
 
                 /* get all the existing windows */
-                client_manage_all();
-                focus_nothing();
+                window_manage_all();
 
                 /* focus what was focused if a wm was already running */
-                if (PROP_GET32(RootWindow(ob_display, ob_screen),
-                               net_active_window, window, &xid) &&
-                    (w = g_hash_table_lookup(window_map, &xid)) &&
-                    WINDOW_IS_CLIENT(w))
+                if (OBT_PROP_GET32(obt_root(ob_screen),
+                                   NET_ACTIVE_WINDOW, WINDOW, &xid) &&
+                    (w = window_find(xid)) && WINDOW_IS_CLIENT(w))
                 {
                     client_focus(WINDOW_AS_CLIENT(w));
                 }
@@ -368,10 +355,12 @@ gint main(gint argc, gchar **argv)
                 }
             }
 
-            reconfigure = FALSE;
-
             ob_set_state(OB_STATE_RUNNING);
 
+            if (!reconfigure && startup_cmd) run_startup_cmd();
+
+            reconfigure = FALSE;
+
             /* look for parsing errors */
             {
                 xmlErrorPtr e = xmlGetLastError();
@@ -386,7 +375,7 @@ gint main(gint argc, gchar **argv)
                 }
             }
 
-            ob_main_loop_run(ob_main_loop);
+            g_main_loop_run(ob_main_loop);
             ob_set_state(reconfigure ?
                          OB_STATE_RECONFIGURING : OB_STATE_EXITING);
 
@@ -395,10 +384,8 @@ gint main(gint argc, gchar **argv)
                 xmlprompt = NULL;
             }
 
-            if (!reconfigure) {
-                dock_remove_all();
-                client_unmanage_all();
-            }
+            if (!reconfigure)
+                window_unmanage_all();
 
             prompt_shutdown(reconfigure);
             menu_shutdown(reconfigure);
@@ -421,24 +408,10 @@ gint main(gint argc, gchar **argv)
             event_shutdown(reconfigure);
             config_shutdown();
             actions_shutdown(reconfigure);
-
-            /* Free the key codes for built in keys */
-            g_free(keys[OB_KEY_RETURN]);
-            g_free(keys[OB_KEY_ESCAPE]);
-            g_free(keys[OB_KEY_LEFT]);
-            g_free(keys[OB_KEY_RIGHT]);
-            g_free(keys[OB_KEY_UP]);
-            g_free(keys[OB_KEY_DOWN]);
-            g_free(keys[OB_KEY_TAB]);
-            g_free(keys[OB_KEY_SPACE]);
-            g_free(keys[OB_KEY_HOME]);
-            g_free(keys[OB_KEY_END]);
-
-            modkeys_shutdown(reconfigure);
         } while (reconfigure);
     }
 
-    XSync(ob_display, FALSE);
+    XSync(obt_display, FALSE);
 
     RrThemeFree(ob_rr_theme);
     RrImageCacheUnref(ob_rr_icons);
@@ -446,11 +419,11 @@ gint main(gint argc, gchar **argv)
 
     session_shutdown(being_replaced);
 
-    XCloseDisplay(ob_display);
-
-    parse_paths_shutdown();
+    obt_display_close();
 
     if (restart) {
+        ob_debug_shutdown();
+        obt_signal_stop();
         if (restart_path != NULL) {
             gint argcp;
             gchar **argvp;
@@ -504,6 +477,11 @@ gint main(gint argc, gchar **argv)
     g_free(ob_sm_id);
     g_free(program_name);
 
+    if (!restart) {
+        ob_debug_shutdown();
+        obt_signal_stop();
+    }
+
     return exitcode;
 }
 
@@ -511,11 +489,11 @@ static void signal_handler(gint signal, gpointer data)
 {
     switch (signal) {
     case SIGUSR1:
-        ob_debug("Caught signal %d. Restarting.\n", signal);
+        ob_debug("Caught signal %d. Restarting.", signal);
         ob_restart();
         break;
     case SIGUSR2:
-        ob_debug("Caught signal %d. Reconfiguring.\n", signal);
+        ob_debug("Caught signal %d. Reconfiguring.", signal);
         ob_reconfigure();
         break;
     case SIGCHLD:
@@ -527,7 +505,7 @@ static void signal_handler(gint signal, gpointer data)
         ob_debug("Caught signal %d. Ignoring.", signal);
         break;
     default:
-        ob_debug("Caught signal %d. Exiting.\n", signal);
+        ob_debug("Caught signal %d. Exiting.", signal);
         /* TERM and INT return a 0 code */
         ob_exit(!(signal == SIGTERM || signal == SIGINT));
     }
@@ -563,8 +541,10 @@ static void print_help(void)
     g_print(_("  --exit              Exit Openbox\n"));
     g_print(_("\nDebugging options:\n"));
     g_print(_("  --sync              Run in synchronous mode\n"));
+    g_print(_("  --startup CMD       Run CMD after starting\n"));
     g_print(_("  --debug             Display debugging output\n"));
     g_print(_("  --debug-focus       Display debugging output for focus handling\n"));
+    g_print(_("  --debug-session     Display debugging output for session management\n"));
     g_print(_("  --debug-xinerama    Split the display into fake xinerama screens\n"));
     g_print(_("\nPlease report bugs at %s\n"), PACKAGE_BUGREPORT);
 }
@@ -580,17 +560,41 @@ static void remove_args(gint *argc, gchar **argv, gint index, gint num)
     *argc -= num;
 }
 
+static void run_startup_cmd(void)
+{
+    gchar **argv = NULL;
+    GError *e = NULL;
+    gboolean ok;
+
+    if (!g_shell_parse_argv(startup_cmd, NULL, &argv, &e)) {
+        g_message("Error parsing startup command: %s",
+                  e->message);
+        g_error_free(e);
+        e = NULL;
+    }
+    ok = g_spawn_async(NULL, argv, NULL,
+                       G_SPAWN_SEARCH_PATH |
+                       G_SPAWN_DO_NOT_REAP_CHILD,
+                       NULL, NULL, NULL, &e);
+    if (!g_shell_parse_argv(startup_cmd, NULL, &argv, &e)) {
+        g_message("Error launching startup command: %s",
+                  e->message);
+        g_error_free(e);
+        e = NULL;
+    }
+}
+
 static void parse_env(void)
 {
     const gchar *id;
 
     /* unset this so we don't pass it on unknowingly */
-    unsetenv("DESKTOP_STARTUP_ID");
+    g_unsetenv("DESKTOP_STARTUP_ID");
 
     /* this is how gnome-session passes in a session client id */
     id = g_getenv("DESKTOP_AUTOSTART_ID");
     if (id) {
-        unsetenv("DESKTOP_AUTOSTART_ID");
+        g_unsetenv("DESKTOP_AUTOSTART_ID");
         if (ob_sm_id) g_free(ob_sm_id);
         ob_sm_id = g_strdup(id);
         ob_debug_type(OB_DEBUG_SM,
@@ -623,17 +627,28 @@ static void parse_args(gint *argc, gchar **argv)
         else if (!strcmp(argv[i], "--sync")) {
             xsync = TRUE;
         }
+        else if (!strcmp(argv[i], "--startup")) {
+            if (i == *argc - 1) /* no args left */
+                g_printerr(_("%s requires an argument\n"), "--startup");
+            else {
+                /* this will be in the current locale encoding, which is
+                   what we want */
+                startup_cmd = argv[i+1];
+                remove_args(argc, argv, i, 2);
+                --i; /* this arg was removed so go back */
+                ob_debug("--startup %s", startup_cmd);
+            }
+        }
         else if (!strcmp(argv[i], "--debug")) {
-            ob_debug_show_output(TRUE);
-            ob_debug_enable(OB_DEBUG_SM, TRUE);
+            ob_debug_enable(OB_DEBUG_NORMAL, TRUE);
             ob_debug_enable(OB_DEBUG_APP_BUGS, TRUE);
         }
         else if (!strcmp(argv[i], "--debug-focus")) {
-            ob_debug_show_output(TRUE);
-            ob_debug_enable(OB_DEBUG_SM, TRUE);
-            ob_debug_enable(OB_DEBUG_APP_BUGS, TRUE);
             ob_debug_enable(OB_DEBUG_FOCUS, TRUE);
         }
+        else if (!strcmp(argv[i], "--debug-session")) {
+            ob_debug_enable(OB_DEBUG_SM, TRUE);
+        }
         else if (!strcmp(argv[i], "--debug-xinerama")) {
             ob_debug_xinerama = TRUE;
         }
@@ -648,13 +663,13 @@ static void parse_args(gint *argc, gchar **argv)
         }
         else if (!strcmp(argv[i], "--config-file")) {
             if (i == *argc - 1) /* no args left */
-                g_printerr(_("--config-file requires an argument\n"));
+                g_printerr(_("%s requires an argument\n"), "--config-file");
             else {
                 /* this will be in the current locale encoding, which is
                    what we want */
                 config_file = argv[i+1];
                 ++i; /* skip the argument */
-                ob_debug("--config-file %s\n", config_file);
+                ob_debug("--config-file %s", config_file);
             }
         }
         else if (!strcmp(argv[i], "--sm-save-file")) {
@@ -665,7 +680,7 @@ static void parse_args(gint *argc, gchar **argv)
                 ob_sm_save_file = g_strdup(argv[i+1]);
                 remove_args(argc, argv, i, 2);
                 --i; /* this arg was removed so go back */
-                ob_debug_type(OB_DEBUG_SM, "--sm-save-file %s\n",
+                ob_debug_type(OB_DEBUG_SM, "--sm-save-file %s",
                               ob_sm_save_file);
             }
         }
@@ -677,7 +692,7 @@ static void parse_args(gint *argc, gchar **argv)
                 ob_sm_id = g_strdup(argv[i+1]);
                 remove_args(argc, argv, i, 2);
                 --i; /* this arg was removed so go back */
-                ob_debug_type(OB_DEBUG_SM, "--sm-client-id %s\n", ob_sm_id);
+                ob_debug_type(OB_DEBUG_SM, "--sm-client-id %s", ob_sm_id);
             }
         }
         else if (!strcmp(argv[i], "--sm-disable")) {
@@ -702,10 +717,10 @@ static Cursor load_cursor(const gchar *name, guint fontval)
     Cursor c = None;
 
 #if USE_XCURSOR
-    c = XcursorLibraryLoadCursor(ob_display, name);
+    c = XcursorLibraryLoadCursor(obt_display, name);
 #endif
     if (c == None)
-        c = XCreateFontCursor(ob_display, fontval);
+        c = XCreateFontCursor(obt_display, fontval);
     return c;
 }
 
@@ -737,14 +752,14 @@ void ob_reconfigure(void)
 void ob_exit(gint code)
 {
     exitcode = code;
-    ob_main_loop_exit(ob_main_loop);
+    g_main_loop_quit(ob_main_loop);
 }
 
 void ob_exit_replace(void)
 {
     exitcode = 0;
     being_replaced = TRUE;
-    ob_main_loop_exit(ob_main_loop);
+    g_main_loop_quit(ob_main_loop);
 }
 
 Cursor ob_cursor(ObCursor cursor)
@@ -753,16 +768,6 @@ Cursor ob_cursor(ObCursor cursor)
     return cursors[cursor];
 }
 
-gboolean ob_keycode_match(KeyCode code, ObKey key)
-{
-    KeyCode *k;
-    
-    g_assert(key < OB_NUM_KEYS);
-    for (k = keys[key]; *k; ++k)
-        if (*k == code) return TRUE;
-    return FALSE;
-}
-
 ObState ob_state(void)
 {
     return state;
